/****************************************************************************
** Meta object code from reading C++ file 'draginfographicsview.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/view/draginfographicsview.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'draginfographicsview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_DragInfoGraphicsView_t {
    QByteArrayData data[24];
    char stringdata0[333];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DragInfoGraphicsView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DragInfoGraphicsView_t qt_meta_stringdata_DragInfoGraphicsView = {
    {
QT_MOC_LITERAL(0, 0, 20), // "DragInfoGraphicsView"
QT_MOC_LITERAL(1, 21, 16), // "signalAngleDelta"
QT_MOC_LITERAL(2, 38, 0), // ""
QT_MOC_LITERAL(3, 39, 5), // "delta"
QT_MOC_LITERAL(4, 45, 21), // "signalsUpdateSchedule"
QT_MOC_LITERAL(5, 67, 18), // "signalScheduleShow"
QT_MOC_LITERAL(6, 86, 6), // "isShow"
QT_MOC_LITERAL(7, 93, 14), // "DSchedule::Ptr"
QT_MOC_LITERAL(8, 108, 3), // "out"
QT_MOC_LITERAL(9, 112, 17), // "signalSceneUpdate"
QT_MOC_LITERAL(10, 130, 19), // "signalSwitchPrePage"
QT_MOC_LITERAL(11, 150, 20), // "signalSwitchNextPage"
QT_MOC_LITERAL(12, 171, 17), // "signalGotoDayView"
QT_MOC_LITERAL(13, 189, 4), // "date"
QT_MOC_LITERAL(14, 194, 14), // "slotDeleteItem"
QT_MOC_LITERAL(15, 209, 17), // "slotSwitchPrePage"
QT_MOC_LITERAL(16, 227, 9), // "focusDate"
QT_MOC_LITERAL(17, 237, 12), // "isSwitchView"
QT_MOC_LITERAL(18, 250, 18), // "slotSwitchNextPage"
QT_MOC_LITERAL(19, 269, 15), // "slotContextMenu"
QT_MOC_LITERAL(20, 285, 11), // "CFocusItem*"
QT_MOC_LITERAL(21, 297, 4), // "item"
QT_MOC_LITERAL(22, 302, 16), // "slotsetNextFocus"
QT_MOC_LITERAL(23, 319, 13) // "touchSlidePos"

    },
    "DragInfoGraphicsView\0signalAngleDelta\0"
    "\0delta\0signalsUpdateSchedule\0"
    "signalScheduleShow\0isShow\0DSchedule::Ptr\0"
    "out\0signalSceneUpdate\0signalSwitchPrePage\0"
    "signalSwitchNextPage\0signalGotoDayView\0"
    "date\0slotDeleteItem\0slotSwitchPrePage\0"
    "focusDate\0isSwitchView\0slotSwitchNextPage\0"
    "slotContextMenu\0CFocusItem*\0item\0"
    "slotsetNextFocus\0touchSlidePos"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DragInfoGraphicsView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      15,   14, // methods
       1,  128, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       8,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   89,    2, 0x06 /* Public */,
       4,    0,   92,    2, 0x06 /* Public */,
       5,    2,   93,    2, 0x06 /* Public */,
       5,    1,   98,    2, 0x26 /* Public | MethodCloned */,
       9,    0,  101,    2, 0x06 /* Public */,
      10,    0,  102,    2, 0x06 /* Public */,
      11,    0,  103,    2, 0x06 /* Public */,
      12,    1,  104,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      14,    0,  107,    2, 0x0a /* Public */,
      15,    2,  108,    2, 0x0a /* Public */,
      15,    1,  113,    2, 0x2a /* Public | MethodCloned */,
      18,    2,  116,    2, 0x0a /* Public */,
      18,    1,  121,    2, 0x2a /* Public | MethodCloned */,
      19,    1,  124,    2, 0x0a /* Public */,
      22,    0,  127,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool, 0x80000000 | 7,    6,    8,
    QMetaType::Void, QMetaType::Bool,    6,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QDate,   13,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QDate, QMetaType::Bool,   16,   17,
    QMetaType::Void, QMetaType::QDate,   16,
    QMetaType::Void, QMetaType::QDate, QMetaType::Bool,   16,   17,
    QMetaType::Void, QMetaType::QDate,   16,
    QMetaType::Void, 0x80000000 | 20,   21,
    QMetaType::Void,

 // properties: name, type, flags
      23, QMetaType::Int, 0x00095003,

       0        // eod
};

void DragInfoGraphicsView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DragInfoGraphicsView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->signalAngleDelta((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->signalsUpdateSchedule(); break;
        case 2: _t->signalScheduleShow((*reinterpret_cast< const bool(*)>(_a[1])),(*reinterpret_cast< const DSchedule::Ptr(*)>(_a[2]))); break;
        case 3: _t->signalScheduleShow((*reinterpret_cast< const bool(*)>(_a[1]))); break;
        case 4: _t->signalSceneUpdate(); break;
        case 5: _t->signalSwitchPrePage(); break;
        case 6: _t->signalSwitchNextPage(); break;
        case 7: _t->signalGotoDayView((*reinterpret_cast< const QDate(*)>(_a[1]))); break;
        case 8: _t->slotDeleteItem(); break;
        case 9: _t->slotSwitchPrePage((*reinterpret_cast< const QDate(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 10: _t->slotSwitchPrePage((*reinterpret_cast< const QDate(*)>(_a[1]))); break;
        case 11: _t->slotSwitchNextPage((*reinterpret_cast< const QDate(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 12: _t->slotSwitchNextPage((*reinterpret_cast< const QDate(*)>(_a[1]))); break;
        case 13: _t->slotContextMenu((*reinterpret_cast< CFocusItem*(*)>(_a[1]))); break;
        case 14: _t->slotsetNextFocus(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< CFocusItem* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DragInfoGraphicsView::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DragInfoGraphicsView::signalAngleDelta)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (DragInfoGraphicsView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DragInfoGraphicsView::signalsUpdateSchedule)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (DragInfoGraphicsView::*)(const bool , const DSchedule::Ptr & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DragInfoGraphicsView::signalScheduleShow)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (DragInfoGraphicsView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DragInfoGraphicsView::signalSceneUpdate)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (DragInfoGraphicsView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DragInfoGraphicsView::signalSwitchPrePage)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (DragInfoGraphicsView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DragInfoGraphicsView::signalSwitchNextPage)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (DragInfoGraphicsView::*)(const QDate & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DragInfoGraphicsView::signalGotoDayView)) {
                *result = 7;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<DragInfoGraphicsView *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< int*>(_v) = _t->getSlidePos(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<DragInfoGraphicsView *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSlidePos(*reinterpret_cast< int*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject DragInfoGraphicsView::staticMetaObject = { {
    QMetaObject::SuperData::link<DGraphicsView::staticMetaObject>(),
    qt_meta_stringdata_DragInfoGraphicsView.data,
    qt_meta_data_DragInfoGraphicsView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DragInfoGraphicsView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DragInfoGraphicsView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DragInfoGraphicsView.stringdata0))
        return static_cast<void*>(this);
    return DGraphicsView::qt_metacast(_clname);
}

int DragInfoGraphicsView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DGraphicsView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void DragInfoGraphicsView::signalAngleDelta(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void DragInfoGraphicsView::signalsUpdateSchedule()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void DragInfoGraphicsView::signalScheduleShow(const bool _t1, const DSchedule::Ptr & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 4
void DragInfoGraphicsView::signalSceneUpdate()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void DragInfoGraphicsView::signalSwitchPrePage()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void DragInfoGraphicsView::signalSwitchNextPage()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void DragInfoGraphicsView::signalGotoDayView(const QDate & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
