/****************************************************************************
** Meta object code from reading C++ file 'dbuscalendar_adaptor.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/dbus/dbuscalendar_adaptor.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dbuscalendar_adaptor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_CalendarAdaptor_t {
    QByteArrayData data[10];
    char stringdata0[383];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CalendarAdaptor_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CalendarAdaptor_t qt_meta_stringdata_CalendarAdaptor = {
    {
QT_MOC_LITERAL(0, 0, 15), // "CalendarAdaptor"
QT_MOC_LITERAL(1, 16, 15), // "D-Bus Interface"
QT_MOC_LITERAL(2, 32, 19), // "com.deepin.Calendar"
QT_MOC_LITERAL(3, 52, 19), // "D-Bus Introspection"
QT_MOC_LITERAL(4, 72, 267), // "  <interface name=\"com.deepi..."
QT_MOC_LITERAL(5, 317, 12), // "ActiveWindow"
QT_MOC_LITERAL(6, 330, 0), // ""
QT_MOC_LITERAL(7, 331, 11), // "RaiseWindow"
QT_MOC_LITERAL(8, 343, 12), // "OpenSchedule"
QT_MOC_LITERAL(9, 356, 3) // "job"

    },
    "CalendarAdaptor\0D-Bus Interface\0"
    "com.deepin.Calendar\0D-Bus Introspection\0"
    "  <interface name=\"com.deepin.Calendar\">\n    <method name=\"ActiveW"
    "indow\">\n    </method>\n    <method name=\"RaiseWindow\">\n    </meth"
    "od>\n    <method name=\"OpenSchedule\">\n      <arg direction=\"in\" t"
    "ype=\"s\" name=\"job\"/>\n    </method>\n  </interface>\n\0"
    "ActiveWindow\0\0RaiseWindow\0OpenSchedule\0"
    "job"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CalendarAdaptor[] = {

 // content:
       8,       // revision
       0,       // classname
       2,   14, // classinfo
       3,   18, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,

 // slots: name, argc, parameters, tag, flags
       5,    0,   33,    6, 0x0a /* Public */,
       7,    0,   34,    6, 0x0a /* Public */,
       8,    1,   35,    6, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    9,

       0        // eod
};

void CalendarAdaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CalendarAdaptor *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->ActiveWindow(); break;
        case 1: _t->RaiseWindow(); break;
        case 2: _t->OpenSchedule((*reinterpret_cast< QString(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject CalendarAdaptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_meta_stringdata_CalendarAdaptor.data,
    qt_meta_data_CalendarAdaptor,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CalendarAdaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CalendarAdaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CalendarAdaptor.stringdata0))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int CalendarAdaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
