/****************************************************************************
** Meta object code from reading C++ file 'jobtypelistview.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/customWidget/jobtypelistview.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'jobtypelistview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_JobTypeListView_t {
    QByteArrayData data[11];
    char stringdata0[189];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_JobTypeListView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_JobTypeListView_t qt_meta_stringdata_JobTypeListView = {
    {
QT_MOC_LITERAL(0, 0, 15), // "JobTypeListView"
QT_MOC_LITERAL(1, 16, 22), // "signalAddStatusChanged"
QT_MOC_LITERAL(2, 39, 0), // ""
QT_MOC_LITERAL(3, 40, 17), // "slotUpdateJobType"
QT_MOC_LITERAL(4, 58, 17), // "slotDeleteJobType"
QT_MOC_LITERAL(5, 76, 19), // "slotAddScheduleType"
QT_MOC_LITERAL(6, 96, 22), // "slotImportScheduleType"
QT_MOC_LITERAL(7, 119, 22), // "slotExportScheduleType"
QT_MOC_LITERAL(8, 142, 13), // "updateJobType"
QT_MOC_LITERAL(9, 156, 21), // "updateCalendarAccount"
QT_MOC_LITERAL(10, 178, 10) // "account_id"

    },
    "JobTypeListView\0signalAddStatusChanged\0"
    "\0slotUpdateJobType\0slotDeleteJobType\0"
    "slotAddScheduleType\0slotImportScheduleType\0"
    "slotExportScheduleType\0updateJobType\0"
    "updateCalendarAccount\0account_id"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_JobTypeListView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   54,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    0,   57,    2, 0x0a /* Public */,
       4,    0,   58,    2, 0x0a /* Public */,
       5,    0,   59,    2, 0x0a /* Public */,
       6,    0,   60,    2, 0x0a /* Public */,
       7,    0,   61,    2, 0x0a /* Public */,
       8,    0,   62,    2, 0x0a /* Public */,
       9,    1,   63,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Bool,
    QMetaType::Void, QMetaType::QString,   10,

       0        // eod
};

void JobTypeListView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<JobTypeListView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->signalAddStatusChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->slotUpdateJobType(); break;
        case 2: _t->slotDeleteJobType(); break;
        case 3: _t->slotAddScheduleType(); break;
        case 4: _t->slotImportScheduleType(); break;
        case 5: _t->slotExportScheduleType(); break;
        case 6: { bool _r = _t->updateJobType();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 7: _t->updateCalendarAccount((*reinterpret_cast< QString(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (JobTypeListView::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&JobTypeListView::signalAddStatusChanged)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject JobTypeListView::staticMetaObject = { {
    QMetaObject::SuperData::link<QTableView::staticMetaObject>(),
    qt_meta_stringdata_JobTypeListView.data,
    qt_meta_data_JobTypeListView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *JobTypeListView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JobTypeListView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_JobTypeListView.stringdata0))
        return static_cast<void*>(this);
    return QTableView::qt_metacast(_clname);
}

int JobTypeListView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTableView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void JobTypeListView::signalAddStatusChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_JobTypeListViewStyle_t {
    QByteArrayData data[1];
    char stringdata0[21];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_JobTypeListViewStyle_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_JobTypeListViewStyle_t qt_meta_stringdata_JobTypeListViewStyle = {
    {
QT_MOC_LITERAL(0, 0, 20) // "JobTypeListViewStyle"

    },
    "JobTypeListViewStyle"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_JobTypeListViewStyle[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void JobTypeListViewStyle::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject JobTypeListViewStyle::staticMetaObject = { {
    QMetaObject::SuperData::link<DStyledItemDelegate::staticMetaObject>(),
    qt_meta_stringdata_JobTypeListViewStyle.data,
    qt_meta_data_JobTypeListViewStyle,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *JobTypeListViewStyle::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JobTypeListViewStyle::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_JobTypeListViewStyle.stringdata0))
        return static_cast<void*>(this);
    return DStyledItemDelegate::qt_metacast(_clname);
}

int JobTypeListViewStyle::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DStyledItemDelegate::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
