/****************************************************************************
** Meta object code from reading C++ file 'yearwindow.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/widget/yearWidget/yearwindow.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'yearwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_CYearWindow_t {
    QByteArrayData data[11];
    char stringdata0[141];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CYearWindow_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CYearWindow_t qt_meta_stringdata_CYearWindow = {
    {
QT_MOC_LITERAL(0, 0, 11), // "CYearWindow"
QT_MOC_LITERAL(1, 12, 22), // "signalsWUpdateShcedule"
QT_MOC_LITERAL(2, 35, 0), // ""
QT_MOC_LITERAL(3, 36, 20), // "signalupdateschedule"
QT_MOC_LITERAL(4, 57, 8), // "slotprev"
QT_MOC_LITERAL(5, 66, 8), // "slotnext"
QT_MOC_LITERAL(6, 75, 9), // "slottoday"
QT_MOC_LITERAL(7, 85, 19), // "slotSetScheduleHide"
QT_MOC_LITERAL(8, 105, 14), // "slotMousePress"
QT_MOC_LITERAL(9, 120, 10), // "selectDate"
QT_MOC_LITERAL(10, 131, 9) // "pressType"

    },
    "CYearWindow\0signalsWUpdateShcedule\0\0"
    "signalupdateschedule\0slotprev\0slotnext\0"
    "slottoday\0slotSetScheduleHide\0"
    "slotMousePress\0selectDate\0pressType"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CYearWindow[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   49,    2, 0x06 /* Public */,
       3,    0,   50,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,   51,    2, 0x08 /* Private */,
       5,    0,   52,    2, 0x08 /* Private */,
       6,    0,   53,    2, 0x08 /* Private */,
       7,    0,   54,    2, 0x0a /* Public */,
       8,    2,   55,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QDate, QMetaType::Int,    9,   10,

       0        // eod
};

void CYearWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CYearWindow *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->signalsWUpdateShcedule(); break;
        case 1: _t->signalupdateschedule(); break;
        case 2: _t->slotprev(); break;
        case 3: _t->slotnext(); break;
        case 4: _t->slottoday(); break;
        case 5: _t->slotSetScheduleHide(); break;
        case 6: _t->slotMousePress((*reinterpret_cast< const QDate(*)>(_a[1])),(*reinterpret_cast< const int(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CYearWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CYearWindow::signalsWUpdateShcedule)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (CYearWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CYearWindow::signalupdateschedule)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject CYearWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<CScheduleBaseWidget::staticMetaObject>(),
    qt_meta_stringdata_CYearWindow.data,
    qt_meta_data_CYearWindow,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CYearWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CYearWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CYearWindow.stringdata0))
        return static_cast<void*>(this);
    return CScheduleBaseWidget::qt_metacast(_clname);
}

int CYearWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = CScheduleBaseWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void CYearWindow::signalsWUpdateShcedule()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void CYearWindow::signalupdateschedule()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
struct qt_meta_stringdata_YearFrame_t {
    QByteArrayData data[5];
    char stringdata0[49];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_YearFrame_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_YearFrame_t qt_meta_stringdata_YearFrame = {
    {
QT_MOC_LITERAL(0, 0, 9), // "YearFrame"
QT_MOC_LITERAL(1, 10, 16), // "signalMousePress"
QT_MOC_LITERAL(2, 27, 0), // ""
QT_MOC_LITERAL(3, 28, 10), // "selectDate"
QT_MOC_LITERAL(4, 39, 9) // "pressType"

    },
    "YearFrame\0signalMousePress\0\0selectDate\0"
    "pressType"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_YearFrame[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   24,    2, 0x06 /* Public */,
       1,    1,   29,    2, 0x26 /* Public | MethodCloned */,

 // signals: parameters
    QMetaType::Void, QMetaType::QDate, QMetaType::Int,    3,    4,
    QMetaType::Void, QMetaType::QDate,    3,

       0        // eod
};

void YearFrame::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<YearFrame *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->signalMousePress((*reinterpret_cast< const QDate(*)>(_a[1])),(*reinterpret_cast< const int(*)>(_a[2]))); break;
        case 1: _t->signalMousePress((*reinterpret_cast< const QDate(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (YearFrame::*)(const QDate & , const int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&YearFrame::signalMousePress)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject YearFrame::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_YearFrame.data,
    qt_meta_data_YearFrame,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *YearFrame::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *YearFrame::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_YearFrame.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int YearFrame::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void YearFrame::signalMousePress(const QDate & _t1, const int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
