/****************************************************************************
** Meta object code from reading C++ file 'daymonthview.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/widget/dayWidget/daymonthview.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'daymonthview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_CDayMonthView_t {
    QByteArrayData data[10];
    char stringdata0[105];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CDayMonthView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CDayMonthView_t qt_meta_stringdata_CDayMonthView = {
    {
QT_MOC_LITERAL(0, 0, 13), // "CDayMonthView"
QT_MOC_LITERAL(1, 14, 16), // "signalIsDragging"
QT_MOC_LITERAL(2, 31, 0), // ""
QT_MOC_LITERAL(3, 32, 5), // "bool&"
QT_MOC_LITERAL(4, 38, 10), // "isDragging"
QT_MOC_LITERAL(5, 49, 22), // "signalChangeSelectDate"
QT_MOC_LITERAL(6, 72, 4), // "date"
QT_MOC_LITERAL(7, 77, 8), // "slotprev"
QT_MOC_LITERAL(8, 86, 8), // "slotnext"
QT_MOC_LITERAL(9, 95, 9) // "slottoday"

    },
    "CDayMonthView\0signalIsDragging\0\0bool&\0"
    "isDragging\0signalChangeSelectDate\0"
    "date\0slotprev\0slotnext\0slottoday"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CDayMonthView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   39,    2, 0x06 /* Public */,
       5,    1,   42,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    0,   45,    2, 0x0a /* Public */,
       8,    0,   46,    2, 0x0a /* Public */,
       9,    0,   47,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, QMetaType::QDate,    6,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void CDayMonthView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CDayMonthView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->signalIsDragging((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->signalChangeSelectDate((*reinterpret_cast< const QDate(*)>(_a[1]))); break;
        case 2: _t->slotprev(); break;
        case 3: _t->slotnext(); break;
        case 4: _t->slottoday(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CDayMonthView::*)(bool & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CDayMonthView::signalIsDragging)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (CDayMonthView::*)(const QDate & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CDayMonthView::signalChangeSelectDate)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject CDayMonthView::staticMetaObject = { {
    QMetaObject::SuperData::link<CustomFrame::staticMetaObject>(),
    qt_meta_stringdata_CDayMonthView.data,
    qt_meta_data_CDayMonthView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CDayMonthView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CDayMonthView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CDayMonthView.stringdata0))
        return static_cast<void*>(this);
    return CustomFrame::qt_metacast(_clname);
}

int CDayMonthView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = CustomFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void CDayMonthView::signalIsDragging(bool & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void CDayMonthView::signalChangeSelectDate(const QDate & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
struct qt_meta_stringdata_CDayMonthWidget_t {
    QByteArrayData data[9];
    char stringdata0[93];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CDayMonthWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CDayMonthWidget_t qt_meta_stringdata_CDayMonthWidget = {
    {
QT_MOC_LITERAL(0, 0, 15), // "CDayMonthWidget"
QT_MOC_LITERAL(1, 16, 22), // "signalChangeSelectDate"
QT_MOC_LITERAL(2, 39, 0), // ""
QT_MOC_LITERAL(3, 40, 4), // "date"
QT_MOC_LITERAL(4, 45, 11), // "cellClicked"
QT_MOC_LITERAL(5, 57, 8), // "QWidget*"
QT_MOC_LITERAL(6, 66, 4), // "cell"
QT_MOC_LITERAL(7, 71, 15), // "setSelectedCell"
QT_MOC_LITERAL(8, 87, 5) // "index"

    },
    "CDayMonthWidget\0signalChangeSelectDate\0"
    "\0date\0cellClicked\0QWidget*\0cell\0"
    "setSelectedCell\0index"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CDayMonthWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   29,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    1,   32,    2, 0x0a /* Public */,
       7,    1,   35,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QDate,    3,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 5,    6,
    QMetaType::Void, QMetaType::Int,    8,

       0        // eod
};

void CDayMonthWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CDayMonthWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->signalChangeSelectDate((*reinterpret_cast< const QDate(*)>(_a[1]))); break;
        case 1: _t->cellClicked((*reinterpret_cast< QWidget*(*)>(_a[1]))); break;
        case 2: _t->setSelectedCell((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWidget* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CDayMonthWidget::*)(const QDate & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CDayMonthWidget::signalChangeSelectDate)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject CDayMonthWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_CDayMonthWidget.data,
    qt_meta_data_CDayMonthWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CDayMonthWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CDayMonthWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CDayMonthWidget.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int CDayMonthWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void CDayMonthWidget::signalChangeSelectDate(const QDate & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
