// This file is autogenerated. Changes will be overwritten.
#include "UVLADIE3JM/moc_attachment.cpp"
#include "UVLADIE3JM/moc_attendee.cpp"
#include "UVLADIE3JM/moc_calendar.cpp"
#include "UVLADIE3JM/moc_calendarplugin.cpp"
#include "UVLADIE3JM/moc_calstorage.cpp"
#include "UVLADIE3JM/moc_conference.cpp"
#include "UVLADIE3JM/moc_event.cpp"
#include "UVLADIE3JM/moc_filestorage.cpp"
#include "UVLADIE3JM/moc_incidence.cpp"
#include "UVLADIE3JM/moc_incidencebase.cpp"
#include "UVLADIE3JM/moc_memorycalendar.cpp"
#include "UVLADIE3JM/moc_person.cpp"
