/****************************************************************************
** Meta object code from reading C++ file 'daccountmanagerservice.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/dbusservice/daccountmanagerservice.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'daccountmanagerservice.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_DAccountManagerService_t {
    QByteArrayData data[27];
    char stringdata0[422];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DAccountManagerService_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DAccountManagerService_t qt_meta_stringdata_DAccountManagerService = {
    {
QT_MOC_LITERAL(0, 0, 22), // "DAccountManagerService"
QT_MOC_LITERAL(1, 23, 15), // "D-Bus Interface"
QT_MOC_LITERAL(2, 39, 45), // "com.deepin.dataserver.Calenda..."
QT_MOC_LITERAL(3, 85, 13), // "accountUpdate"
QT_MOC_LITERAL(4, 99, 0), // ""
QT_MOC_LITERAL(5, 100, 14), // "getAccountList"
QT_MOC_LITERAL(6, 115, 9), // "remindJob"
QT_MOC_LITERAL(7, 125, 9), // "accountID"
QT_MOC_LITERAL(8, 135, 7), // "alarmID"
QT_MOC_LITERAL(9, 143, 15), // "updateRemindJob"
QT_MOC_LITERAL(10, 159, 7), // "isClear"
QT_MOC_LITERAL(11, 167, 16), // "notifyMsgHanding"
QT_MOC_LITERAL(12, 184, 12), // "operationNum"
QT_MOC_LITERAL(13, 197, 19), // "downloadByAccountID"
QT_MOC_LITERAL(14, 217, 24), // "uploadNetWorkAccountData"
QT_MOC_LITERAL(15, 242, 26), // "getCalendarGeneralSettings"
QT_MOC_LITERAL(16, 269, 26), // "setCalendarGeneralSettings"
QT_MOC_LITERAL(17, 296, 5), // "cgSet"
QT_MOC_LITERAL(18, 302, 14), // "calendarIsShow"
QT_MOC_LITERAL(19, 317, 6), // "isShow"
QT_MOC_LITERAL(20, 324, 5), // "login"
QT_MOC_LITERAL(21, 330, 6), // "logout"
QT_MOC_LITERAL(22, 337, 12), // "isSupportUid"
QT_MOC_LITERAL(23, 350, 14), // "firstDayOfWeek"
QT_MOC_LITERAL(24, 365, 14), // "timeFormatType"
QT_MOC_LITERAL(25, 380, 20), // "firstDayOfWeekSource"
QT_MOC_LITERAL(26, 401, 20) // "timeFormatTypeSource"

    },
    "DAccountManagerService\0D-Bus Interface\0"
    "com.deepin.dataserver.Calendar.AccountManager\0"
    "accountUpdate\0\0getAccountList\0remindJob\0"
    "accountID\0alarmID\0updateRemindJob\0"
    "isClear\0notifyMsgHanding\0operationNum\0"
    "downloadByAccountID\0uploadNetWorkAccountData\0"
    "getCalendarGeneralSettings\0"
    "setCalendarGeneralSettings\0cgSet\0"
    "calendarIsShow\0isShow\0login\0logout\0"
    "isSupportUid\0firstDayOfWeek\0timeFormatType\0"
    "firstDayOfWeekSource\0timeFormatTypeSource"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DAccountManagerService[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      13,   16, // methods
       4,  112, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,   81,    4, 0x46 /* Public | isScriptable */,

 // slots: name, argc, parameters, tag, flags
       5,    0,   82,    4, 0x4a /* Public | isScriptable */,
       6,    2,   83,    4, 0x4a /* Public | isScriptable */,
       9,    1,   88,    4, 0x4a /* Public | isScriptable */,
      11,    3,   91,    4, 0x4a /* Public | isScriptable */,
      13,    1,   98,    4, 0x4a /* Public | isScriptable */,
      14,    0,  101,    4, 0x4a /* Public | isScriptable */,
      15,    0,  102,    4, 0x4a /* Public | isScriptable */,
      16,    1,  103,    4, 0x4a /* Public | isScriptable */,
      18,    1,  106,    4, 0x4a /* Public | isScriptable */,
      20,    0,  109,    4, 0x4a /* Public | isScriptable */,
      21,    0,  110,    4, 0x4a /* Public | isScriptable */,
      22,    0,  111,    4, 0x4a /* Public | isScriptable */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::QString,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    7,    8,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::Int,    7,    8,   12,
    QMetaType::Void, QMetaType::QString,    7,
    QMetaType::Void,
    QMetaType::QString,
    QMetaType::Void, QMetaType::QString,   17,
    QMetaType::Void, QMetaType::Bool,   19,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Bool,

 // properties: name, type, flags
      23, QMetaType::Int, 0x00095103,
      24, QMetaType::Int, 0x00095103,
      25, QMetaType::Int, 0x00095103,
      26, QMetaType::Int, 0x00095103,

       0        // eod
};

void DAccountManagerService::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DAccountManagerService *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->accountUpdate(); break;
        case 1: { QString _r = _t->getAccountList();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 2: _t->remindJob((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 3: _t->updateRemindJob((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->notifyMsgHanding((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const qint32(*)>(_a[3]))); break;
        case 5: _t->downloadByAccountID((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 6: _t->uploadNetWorkAccountData(); break;
        case 7: { QString _r = _t->getCalendarGeneralSettings();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 8: _t->setCalendarGeneralSettings((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 9: _t->calendarIsShow((*reinterpret_cast< const bool(*)>(_a[1]))); break;
        case 10: _t->login(); break;
        case 11: _t->logout(); break;
        case 12: { bool _r = _t->isSupportUid();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DAccountManagerService::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DAccountManagerService::accountUpdate)) {
                *result = 0;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<DAccountManagerService *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< int*>(_v) = _t->getfirstDayOfWeek(); break;
        case 1: *reinterpret_cast< int*>(_v) = _t->getTimeFormatType(); break;
        case 2: *reinterpret_cast< int*>(_v) = _t->getFirstDayOfWeekSource(); break;
        case 3: *reinterpret_cast< int*>(_v) = _t->getTimeFormatTypeSource(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<DAccountManagerService *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setFirstDayOfWeek(*reinterpret_cast< int*>(_v)); break;
        case 1: _t->setTimeFormatType(*reinterpret_cast< int*>(_v)); break;
        case 2: _t->setFirstDayOfWeekSource(*reinterpret_cast< int*>(_v)); break;
        case 3: _t->setTimeFormatTypeSource(*reinterpret_cast< int*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject DAccountManagerService::staticMetaObject = { {
    QMetaObject::SuperData::link<DServiceBase::staticMetaObject>(),
    qt_meta_stringdata_DAccountManagerService.data,
    qt_meta_data_DAccountManagerService,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DAccountManagerService::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DAccountManagerService::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DAccountManagerService.stringdata0))
        return static_cast<void*>(this);
    return DServiceBase::qt_metacast(_clname);
}

int DAccountManagerService::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DServiceBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 13;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 4;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void DAccountManagerService::accountUpdate()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
