/****************************************************************************
** Meta object code from reading C++ file 'scheduletypeeditdlg.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/dialog/scheduletypeeditdlg.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'scheduletypeeditdlg.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ScheduleTypeEditDlg_t {
    QByteArrayData data[11];
    char stringdata0[169];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ScheduleTypeEditDlg_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ScheduleTypeEditDlg_t qt_meta_stringdata_ScheduleTypeEditDlg = {
    {
QT_MOC_LITERAL(0, 0, 19), // "ScheduleTypeEditDlg"
QT_MOC_LITERAL(1, 20, 25), // "signalRefreshScheduleType"
QT_MOC_LITERAL(2, 46, 0), // ""
QT_MOC_LITERAL(3, 47, 19), // "slotEditTextChanged"
QT_MOC_LITERAL(4, 67, 16), // "slotFocusChanged"
QT_MOC_LITERAL(5, 84, 7), // "onFocus"
QT_MOC_LITERAL(6, 92, 13), // "slotBtnCancel"
QT_MOC_LITERAL(7, 106, 11), // "slotBtnNext"
QT_MOC_LITERAL(8, 118, 19), // "slotEditingFinished"
QT_MOC_LITERAL(9, 138, 10), // "getIcsFile"
QT_MOC_LITERAL(10, 149, 19) // "slotCheckConfirmBtn"

    },
    "ScheduleTypeEditDlg\0signalRefreshScheduleType\0"
    "\0slotEditTextChanged\0slotFocusChanged\0"
    "onFocus\0slotBtnCancel\0slotBtnNext\0"
    "slotEditingFinished\0getIcsFile\0"
    "slotCheckConfirmBtn"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ScheduleTypeEditDlg[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   54,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    1,   55,    2, 0x0a /* Public */,
       4,    1,   58,    2, 0x0a /* Public */,
       6,    0,   61,    2, 0x0a /* Public */,
       7,    0,   62,    2, 0x0a /* Public */,
       8,    0,   63,    2, 0x0a /* Public */,
       9,    0,   64,    2, 0x0a /* Public */,
      10,    0,   65,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Bool,    5,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::QString,
    QMetaType::Void,

       0        // eod
};

void ScheduleTypeEditDlg::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ScheduleTypeEditDlg *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->signalRefreshScheduleType(); break;
        case 1: _t->slotEditTextChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->slotFocusChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->slotBtnCancel(); break;
        case 4: _t->slotBtnNext(); break;
        case 5: _t->slotEditingFinished(); break;
        case 6: { QString _r = _t->getIcsFile();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 7: _t->slotCheckConfirmBtn(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ScheduleTypeEditDlg::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScheduleTypeEditDlg::signalRefreshScheduleType)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ScheduleTypeEditDlg::staticMetaObject = { {
    QMetaObject::SuperData::link<DDialog::staticMetaObject>(),
    qt_meta_stringdata_ScheduleTypeEditDlg.data,
    qt_meta_data_ScheduleTypeEditDlg,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ScheduleTypeEditDlg::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ScheduleTypeEditDlg::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ScheduleTypeEditDlg.stringdata0))
        return static_cast<void*>(this);
    return DDialog::qt_metacast(_clname);
}

int ScheduleTypeEditDlg::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void ScheduleTypeEditDlg::signalRefreshScheduleType()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
