/****************************************************************************
** Meta object code from reading C++ file 'timeedit.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/customWidget/timeedit.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'timeedit.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_CTimeEdit_t {
    QByteArrayData data[12];
    char stringdata0[147];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CTimeEdit_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CTimeEdit_t qt_meta_stringdata_CTimeEdit = {
    {
QT_MOC_LITERAL(0, 0, 9), // "CTimeEdit"
QT_MOC_LITERAL(1, 10, 14), // "signalFocusOut"
QT_MOC_LITERAL(2, 25, 0), // ""
QT_MOC_LITERAL(3, 26, 21), // "signaleditingFinished"
QT_MOC_LITERAL(4, 48, 13), // "setTimeFormat"
QT_MOC_LITERAL(5, 62, 5), // "value"
QT_MOC_LITERAL(6, 68, 13), // "slotFocusDraw"
QT_MOC_LITERAL(7, 82, 9), // "showFocus"
QT_MOC_LITERAL(8, 92, 16), // "slotSetPlainText"
QT_MOC_LITERAL(9, 109, 3), // "arg"
QT_MOC_LITERAL(10, 113, 13), // "slotActivated"
QT_MOC_LITERAL(11, 127, 19) // "slotEditingFinished"

    },
    "CTimeEdit\0signalFocusOut\0\0"
    "signaleditingFinished\0setTimeFormat\0"
    "value\0slotFocusDraw\0showFocus\0"
    "slotSetPlainText\0arg\0slotActivated\0"
    "slotEditingFinished"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CTimeEdit[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   49,    2, 0x06 /* Public */,
       3,    0,   50,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    1,   51,    2, 0x0a /* Public */,
       6,    1,   54,    2, 0x0a /* Public */,
       8,    1,   57,    2, 0x0a /* Public */,
      10,    1,   60,    2, 0x0a /* Public */,
      11,    0,   63,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Bool,    7,
    QMetaType::Void, QMetaType::QString,    9,
    QMetaType::Void, QMetaType::QString,    9,
    QMetaType::Void,

       0        // eod
};

void CTimeEdit::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CTimeEdit *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->signalFocusOut(); break;
        case 1: _t->signaleditingFinished(); break;
        case 2: _t->setTimeFormat((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->slotFocusDraw((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->slotSetPlainText((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->slotActivated((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 6: _t->slotEditingFinished(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CTimeEdit::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CTimeEdit::signalFocusOut)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (CTimeEdit::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CTimeEdit::signaleditingFinished)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject CTimeEdit::staticMetaObject = { {
    QMetaObject::SuperData::link<DComboBox::staticMetaObject>(),
    qt_meta_stringdata_CTimeEdit.data,
    qt_meta_data_CTimeEdit,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CTimeEdit::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CTimeEdit::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CTimeEdit.stringdata0))
        return static_cast<void*>(this);
    return DComboBox::qt_metacast(_clname);
}

int CTimeEdit::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DComboBox::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void CTimeEdit::signalFocusOut()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void CTimeEdit::signaleditingFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
