/****************************************************************************
** Meta object code from reading C++ file 'daccountmanagemodule.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/calendarDataManager/daccountmanagemodule.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'daccountmanagemodule.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_DAccountManageModule_t {
    QByteArrayData data[15];
    char stringdata0[236];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DAccountManageModule_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DAccountManageModule_t qt_meta_stringdata_DAccountManageModule = {
    {
QT_MOC_LITERAL(0, 0, 20), // "DAccountManageModule"
QT_MOC_LITERAL(1, 21, 20), // "firstDayOfWeekChange"
QT_MOC_LITERAL(2, 42, 0), // ""
QT_MOC_LITERAL(3, 43, 20), // "timeFormatTypeChange"
QT_MOC_LITERAL(4, 64, 23), // "signalLoginStatusChange"
QT_MOC_LITERAL(5, 88, 18), // "slotFirstDayOfWeek"
QT_MOC_LITERAL(6, 107, 8), // "firstDay"
QT_MOC_LITERAL(7, 116, 18), // "slotTimeFormatType"
QT_MOC_LITERAL(8, 135, 8), // "timeType"
QT_MOC_LITERAL(9, 144, 24), // "slotUidLoginStatueChange"
QT_MOC_LITERAL(10, 169, 6), // "status"
QT_MOC_LITERAL(11, 176, 18), // "slotSwitcherChange"
QT_MOC_LITERAL(12, 195, 5), // "state"
QT_MOC_LITERAL(13, 201, 17), // "slotSettingChange"
QT_MOC_LITERAL(14, 219, 16) // "slotClientIsOpen"

    },
    "DAccountManageModule\0firstDayOfWeekChange\0"
    "\0timeFormatTypeChange\0signalLoginStatusChange\0"
    "slotFirstDayOfWeek\0firstDay\0"
    "slotTimeFormatType\0timeType\0"
    "slotUidLoginStatueChange\0status\0"
    "slotSwitcherChange\0state\0slotSettingChange\0"
    "slotClientIsOpen"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DAccountManageModule[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   59,    2, 0x06 /* Public */,
       3,    0,   60,    2, 0x06 /* Public */,
       4,    0,   61,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    1,   62,    2, 0x0a /* Public */,
       7,    1,   65,    2, 0x0a /* Public */,
       9,    1,   68,    2, 0x0a /* Public */,
      11,    1,   71,    2, 0x0a /* Public */,
      13,    0,   74,    2, 0x0a /* Public */,
      14,    0,   75,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    6,
    QMetaType::Void, QMetaType::Int,    8,
    QMetaType::Void, QMetaType::Int,   10,
    QMetaType::Void, QMetaType::Bool,   12,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void DAccountManageModule::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DAccountManageModule *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->firstDayOfWeekChange(); break;
        case 1: _t->timeFormatTypeChange(); break;
        case 2: _t->signalLoginStatusChange(); break;
        case 3: _t->slotFirstDayOfWeek((*reinterpret_cast< const int(*)>(_a[1]))); break;
        case 4: _t->slotTimeFormatType((*reinterpret_cast< const int(*)>(_a[1]))); break;
        case 5: _t->slotUidLoginStatueChange((*reinterpret_cast< const int(*)>(_a[1]))); break;
        case 6: _t->slotSwitcherChange((*reinterpret_cast< const bool(*)>(_a[1]))); break;
        case 7: _t->slotSettingChange(); break;
        case 8: _t->slotClientIsOpen(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DAccountManageModule::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DAccountManageModule::firstDayOfWeekChange)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (DAccountManageModule::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DAccountManageModule::timeFormatTypeChange)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (DAccountManageModule::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DAccountManageModule::signalLoginStatusChange)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DAccountManageModule::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_DAccountManageModule.data,
    qt_meta_data_DAccountManageModule,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DAccountManageModule::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DAccountManageModule::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DAccountManageModule.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int DAccountManageModule::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void DAccountManageModule::firstDayOfWeekChange()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void DAccountManageModule::timeFormatTypeChange()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void DAccountManageModule::signalLoginStatusChange()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
