/****************************************************************************
** Meta object code from reading C++ file 'dbusaccountrequest.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/dbus/dbusaccountrequest.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dbusaccountrequest.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_DbusAccountRequest_t {
    QByteArrayData data[17];
    char stringdata0[361];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DbusAccountRequest_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DbusAccountRequest_t qt_meta_stringdata_DbusAccountRequest = {
    {
QT_MOC_LITERAL(0, 0, 18), // "DbusAccountRequest"
QT_MOC_LITERAL(1, 19, 26), // "signalCreateScheduleFinish"
QT_MOC_LITERAL(2, 46, 0), // ""
QT_MOC_LITERAL(3, 47, 10), // "scheduleID"
QT_MOC_LITERAL(4, 58, 26), // "signalGetAccountInfoFinish"
QT_MOC_LITERAL(5, 85, 13), // "DAccount::Ptr"
QT_MOC_LITERAL(6, 99, 23), // "signalGetScheduleFinish"
QT_MOC_LITERAL(7, 123, 14), // "DSchedule::Ptr"
QT_MOC_LITERAL(8, 138, 31), // "signalGetScheduleTypeListFinish"
QT_MOC_LITERAL(9, 170, 19), // "DScheduleType::List"
QT_MOC_LITERAL(10, 190, 27), // "signalGetScheduleListFinish"
QT_MOC_LITERAL(11, 218, 27), // "QMap<QDate,DSchedule::List>"
QT_MOC_LITERAL(12, 246, 30), // "signalSearchScheduleListFinish"
QT_MOC_LITERAL(13, 277, 24), // "signalGetSysColorsFinish"
QT_MOC_LITERAL(14, 302, 16), // "DTypeColor::List"
QT_MOC_LITERAL(15, 319, 16), // "slotCallFinished"
QT_MOC_LITERAL(16, 336, 24) // "CDBusPendingCallWatcher*"

    },
    "DbusAccountRequest\0signalCreateScheduleFinish\0"
    "\0scheduleID\0signalGetAccountInfoFinish\0"
    "DAccount::Ptr\0signalGetScheduleFinish\0"
    "DSchedule::Ptr\0signalGetScheduleTypeListFinish\0"
    "DScheduleType::List\0signalGetScheduleListFinish\0"
    "QMap<QDate,DSchedule::List>\0"
    "signalSearchScheduleListFinish\0"
    "signalGetSysColorsFinish\0DTypeColor::List\0"
    "slotCallFinished\0CDBusPendingCallWatcher*"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DbusAccountRequest[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       7,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   54,    2, 0x06 /* Public */,
       4,    1,   57,    2, 0x06 /* Public */,
       6,    1,   60,    2, 0x06 /* Public */,
       8,    1,   63,    2, 0x06 /* Public */,
      10,    1,   66,    2, 0x06 /* Public */,
      12,    1,   69,    2, 0x06 /* Public */,
      13,    1,   72,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      15,    1,   75,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, 0x80000000 | 5,    2,
    QMetaType::Void, 0x80000000 | 7,    2,
    QMetaType::Void, 0x80000000 | 9,    2,
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void, 0x80000000 | 14,    2,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 16,    2,

       0        // eod
};

void DbusAccountRequest::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DbusAccountRequest *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->signalCreateScheduleFinish((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->signalGetAccountInfoFinish((*reinterpret_cast< DAccount::Ptr(*)>(_a[1]))); break;
        case 2: _t->signalGetScheduleFinish((*reinterpret_cast< const DSchedule::Ptr(*)>(_a[1]))); break;
        case 3: _t->signalGetScheduleTypeListFinish((*reinterpret_cast< DScheduleType::List(*)>(_a[1]))); break;
        case 4: _t->signalGetScheduleListFinish((*reinterpret_cast< QMap<QDate,DSchedule::List>(*)>(_a[1]))); break;
        case 5: _t->signalSearchScheduleListFinish((*reinterpret_cast< QMap<QDate,DSchedule::List>(*)>(_a[1]))); break;
        case 6: _t->signalGetSysColorsFinish((*reinterpret_cast< DTypeColor::List(*)>(_a[1]))); break;
        case 7: _t->slotCallFinished((*reinterpret_cast< CDBusPendingCallWatcher*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< CDBusPendingCallWatcher* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DbusAccountRequest::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DbusAccountRequest::signalCreateScheduleFinish)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (DbusAccountRequest::*)(DAccount::Ptr );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DbusAccountRequest::signalGetAccountInfoFinish)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (DbusAccountRequest::*)(const DSchedule::Ptr & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DbusAccountRequest::signalGetScheduleFinish)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (DbusAccountRequest::*)(DScheduleType::List );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DbusAccountRequest::signalGetScheduleTypeListFinish)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (DbusAccountRequest::*)(QMap<QDate,DSchedule::List> );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DbusAccountRequest::signalGetScheduleListFinish)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (DbusAccountRequest::*)(QMap<QDate,DSchedule::List> );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DbusAccountRequest::signalSearchScheduleListFinish)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (DbusAccountRequest::*)(DTypeColor::List );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DbusAccountRequest::signalGetSysColorsFinish)) {
                *result = 6;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DbusAccountRequest::staticMetaObject = { {
    QMetaObject::SuperData::link<DbusRequestBase::staticMetaObject>(),
    qt_meta_stringdata_DbusAccountRequest.data,
    qt_meta_data_DbusAccountRequest,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DbusAccountRequest::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DbusAccountRequest::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DbusAccountRequest.stringdata0))
        return static_cast<void*>(this);
    return DbusRequestBase::qt_metacast(_clname);
}

int DbusAccountRequest::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DbusRequestBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void DbusAccountRequest::signalCreateScheduleFinish(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void DbusAccountRequest::signalGetAccountInfoFinish(DAccount::Ptr _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void DbusAccountRequest::signalGetScheduleFinish(const DSchedule::Ptr & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void DbusAccountRequest::signalGetScheduleTypeListFinish(DScheduleType::List _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void DbusAccountRequest::signalGetScheduleListFinish(QMap<QDate,DSchedule::List> _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void DbusAccountRequest::signalSearchScheduleListFinish(QMap<QDate,DSchedule::List> _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void DbusAccountRequest::signalGetSysColorsFinish(DTypeColor::List _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
