/****************************************************************************
** Meta object code from reading C++ file 'attendee.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/attendee.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'attendee.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_KCalendarCore__Attendee_t {
    QByteArrayData data[32];
    char stringdata0[284];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KCalendarCore__Attendee_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KCalendarCore__Attendee_t qt_meta_stringdata_KCalendarCore__Attendee = {
    {
QT_MOC_LITERAL(0, 0, 23), // "KCalendarCore::Attendee"
QT_MOC_LITERAL(1, 24, 6), // "isNull"
QT_MOC_LITERAL(2, 31, 4), // "name"
QT_MOC_LITERAL(3, 36, 8), // "fullName"
QT_MOC_LITERAL(4, 45, 5), // "email"
QT_MOC_LITERAL(5, 51, 4), // "role"
QT_MOC_LITERAL(6, 56, 4), // "Role"
QT_MOC_LITERAL(7, 61, 3), // "uid"
QT_MOC_LITERAL(8, 65, 6), // "status"
QT_MOC_LITERAL(9, 72, 8), // "PartStat"
QT_MOC_LITERAL(10, 81, 6), // "cuType"
QT_MOC_LITERAL(11, 88, 6), // "CuType"
QT_MOC_LITERAL(12, 95, 4), // "rsvp"
QT_MOC_LITERAL(13, 100, 8), // "delegate"
QT_MOC_LITERAL(14, 109, 9), // "delegator"
QT_MOC_LITERAL(15, 119, 11), // "NeedsAction"
QT_MOC_LITERAL(16, 131, 8), // "Accepted"
QT_MOC_LITERAL(17, 140, 8), // "Declined"
QT_MOC_LITERAL(18, 149, 9), // "Tentative"
QT_MOC_LITERAL(19, 159, 9), // "Delegated"
QT_MOC_LITERAL(20, 169, 9), // "Completed"
QT_MOC_LITERAL(21, 179, 9), // "InProcess"
QT_MOC_LITERAL(22, 189, 4), // "None"
QT_MOC_LITERAL(23, 194, 14), // "ReqParticipant"
QT_MOC_LITERAL(24, 209, 14), // "OptParticipant"
QT_MOC_LITERAL(25, 224, 14), // "NonParticipant"
QT_MOC_LITERAL(26, 239, 5), // "Chair"
QT_MOC_LITERAL(27, 245, 10), // "Individual"
QT_MOC_LITERAL(28, 256, 5), // "Group"
QT_MOC_LITERAL(29, 262, 8), // "Resource"
QT_MOC_LITERAL(30, 271, 4), // "Room"
QT_MOC_LITERAL(31, 276, 7) // "Unknown"

    },
    "KCalendarCore::Attendee\0isNull\0name\0"
    "fullName\0email\0role\0Role\0uid\0status\0"
    "PartStat\0cuType\0CuType\0rsvp\0delegate\0"
    "delegator\0NeedsAction\0Accepted\0Declined\0"
    "Tentative\0Delegated\0Completed\0InProcess\0"
    "None\0ReqParticipant\0OptParticipant\0"
    "NonParticipant\0Chair\0Individual\0Group\0"
    "Resource\0Room\0Unknown"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KCalendarCore__Attendee[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
      11,   14, // properties
       3,   47, // enums/sets
       0,    0, // constructors
       4,       // flags
       0,       // signalCount

 // properties: name, type, flags
       1, QMetaType::Bool, 0x00095001,
       2, QMetaType::QString, 0x00095103,
       3, QMetaType::QString, 0x00095001,
       4, QMetaType::QString, 0x00095103,
       5, 0x80000000 | 6, 0x0009510b,
       7, QMetaType::QString, 0x00095103,
       8, 0x80000000 | 9, 0x0009510b,
      10, 0x80000000 | 11, 0x0009510b,
      12, QMetaType::Bool, 0x00095003,
      13, QMetaType::QString, 0x00095103,
      14, QMetaType::QString, 0x00095103,

 // enums: name, alias, flags, count, data
       9,    9, 0x0,    8,   62,
       6,    6, 0x0,    4,   78,
      11,   11, 0x0,    5,   86,

 // enum data: key, value
      15, uint(KCalendarCore::Attendee::NeedsAction),
      16, uint(KCalendarCore::Attendee::Accepted),
      17, uint(KCalendarCore::Attendee::Declined),
      18, uint(KCalendarCore::Attendee::Tentative),
      19, uint(KCalendarCore::Attendee::Delegated),
      20, uint(KCalendarCore::Attendee::Completed),
      21, uint(KCalendarCore::Attendee::InProcess),
      22, uint(KCalendarCore::Attendee::None),
      23, uint(KCalendarCore::Attendee::ReqParticipant),
      24, uint(KCalendarCore::Attendee::OptParticipant),
      25, uint(KCalendarCore::Attendee::NonParticipant),
      26, uint(KCalendarCore::Attendee::Chair),
      27, uint(KCalendarCore::Attendee::Individual),
      28, uint(KCalendarCore::Attendee::Group),
      29, uint(KCalendarCore::Attendee::Resource),
      30, uint(KCalendarCore::Attendee::Room),
      31, uint(KCalendarCore::Attendee::Unknown),

       0        // eod
};

void KCalendarCore::Attendee::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{

#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty) {
        auto *_t = reinterpret_cast<Attendee *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->isNull(); break;
        case 1: *reinterpret_cast< QString*>(_v) = _t->name(); break;
        case 2: *reinterpret_cast< QString*>(_v) = _t->fullName(); break;
        case 3: *reinterpret_cast< QString*>(_v) = _t->email(); break;
        case 4: *reinterpret_cast< Role*>(_v) = _t->role(); break;
        case 5: *reinterpret_cast< QString*>(_v) = _t->uid(); break;
        case 6: *reinterpret_cast< PartStat*>(_v) = _t->status(); break;
        case 7: *reinterpret_cast< CuType*>(_v) = _t->cuType(); break;
        case 8: *reinterpret_cast< bool*>(_v) = _t->RSVP(); break;
        case 9: *reinterpret_cast< QString*>(_v) = _t->delegate(); break;
        case 10: *reinterpret_cast< QString*>(_v) = _t->delegator(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = reinterpret_cast<Attendee *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setName(*reinterpret_cast< QString*>(_v)); break;
        case 3: _t->setEmail(*reinterpret_cast< QString*>(_v)); break;
        case 4: _t->setRole(*reinterpret_cast< Role*>(_v)); break;
        case 5: _t->setUid(*reinterpret_cast< QString*>(_v)); break;
        case 6: _t->setStatus(*reinterpret_cast< PartStat*>(_v)); break;
        case 7: _t->setCuType(*reinterpret_cast< CuType*>(_v)); break;
        case 8: _t->setRSVP(*reinterpret_cast< bool*>(_v)); break;
        case 9: _t->setDelegate(*reinterpret_cast< QString*>(_v)); break;
        case 10: _t->setDelegator(*reinterpret_cast< QString*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject KCalendarCore::Attendee::staticMetaObject = { {
    nullptr,
    qt_meta_stringdata_KCalendarCore__Attendee.data,
    qt_meta_data_KCalendarCore__Attendee,
    qt_static_metacall,
    nullptr,
    nullptr
} };

QT_WARNING_POP
QT_END_MOC_NAMESPACE
