/****************************************************************************
** Meta object code from reading C++ file 'syncoperation.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/synchronization/syncoperation.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'syncoperation.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Syncoperation_t {
    QByteArrayData data[15];
    char stringdata0[187];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Syncoperation_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Syncoperation_t qt_meta_stringdata_Syncoperation = {
    {
QT_MOC_LITERAL(0, 0, 13), // "Syncoperation"
QT_MOC_LITERAL(1, 14, 23), // "signalLoginStatusChange"
QT_MOC_LITERAL(2, 38, 0), // ""
QT_MOC_LITERAL(3, 39, 5), // "staus"
QT_MOC_LITERAL(4, 45, 11), // "LoginStatus"
QT_MOC_LITERAL(5, 57, 4), // "data"
QT_MOC_LITERAL(6, 62, 14), // "SwitcherChange"
QT_MOC_LITERAL(7, 77, 5), // "state"
QT_MOC_LITERAL(8, 83, 12), // "slotDbusCall"
QT_MOC_LITERAL(9, 96, 12), // "QDBusMessage"
QT_MOC_LITERAL(10, 109, 3), // "msg"
QT_MOC_LITERAL(11, 113, 19), // "onPropertiesChanged"
QT_MOC_LITERAL(12, 133, 13), // "interfaceName"
QT_MOC_LITERAL(13, 147, 17), // "changedProperties"
QT_MOC_LITERAL(14, 165, 21) // "invalidatedProperties"

    },
    "Syncoperation\0signalLoginStatusChange\0"
    "\0staus\0LoginStatus\0data\0SwitcherChange\0"
    "state\0slotDbusCall\0QDBusMessage\0msg\0"
    "onPropertiesChanged\0interfaceName\0"
    "changedProperties\0invalidatedProperties"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Syncoperation[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   39,    2, 0x06 /* Public */,
       4,    1,   42,    2, 0x06 /* Public */,
       6,    1,   45,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       8,    1,   48,    2, 0x08 /* Private */,
      11,    3,   51,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Bool,    7,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void, QMetaType::QString, QMetaType::QVariantMap, QMetaType::QStringList,   12,   13,   14,

       0        // eod
};

void Syncoperation::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Syncoperation *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->signalLoginStatusChange((*reinterpret_cast< const int(*)>(_a[1]))); break;
        case 1: _t->LoginStatus((*reinterpret_cast< const int(*)>(_a[1]))); break;
        case 2: _t->SwitcherChange((*reinterpret_cast< const bool(*)>(_a[1]))); break;
        case 3: _t->slotDbusCall((*reinterpret_cast< const QDBusMessage(*)>(_a[1]))); break;
        case 4: _t->onPropertiesChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QVariantMap(*)>(_a[2])),(*reinterpret_cast< const QStringList(*)>(_a[3]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusMessage >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (Syncoperation::*)(const int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Syncoperation::signalLoginStatusChange)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (Syncoperation::*)(const int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Syncoperation::LoginStatus)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (Syncoperation::*)(const bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Syncoperation::SwitcherChange)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Syncoperation::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_Syncoperation.data,
    qt_meta_data_Syncoperation,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Syncoperation::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Syncoperation::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Syncoperation.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Syncoperation::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void Syncoperation::signalLoginStatusChange(const int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Syncoperation::LoginStatus(const int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Syncoperation::SwitcherChange(const bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
