/****************************************************************************
** Meta object code from reading C++ file 'event.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/event.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'event.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_KCalendarCore__Event_t {
    QByteArrayData data[7];
    char stringdata0[107];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KCalendarCore__Event_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KCalendarCore__Event_t qt_meta_stringdata_KCalendarCore__Event = {
    {
QT_MOC_LITERAL(0, 0, 20), // "KCalendarCore::Event"
QT_MOC_LITERAL(1, 21, 5), // "dtEnd"
QT_MOC_LITERAL(2, 27, 12), // "transparency"
QT_MOC_LITERAL(3, 40, 34), // "KCalendarCore::Event::Transpa..."
QT_MOC_LITERAL(4, 75, 12), // "Transparency"
QT_MOC_LITERAL(5, 88, 6), // "Opaque"
QT_MOC_LITERAL(6, 95, 11) // "Transparent"

    },
    "KCalendarCore::Event\0dtEnd\0transparency\0"
    "KCalendarCore::Event::Transparency\0"
    "Transparency\0Opaque\0Transparent"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KCalendarCore__Event[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       2,   14, // properties
       1,   20, // enums/sets
       0,    0, // constructors
       4,       // flags
       0,       // signalCount

 // properties: name, type, flags
       1, QMetaType::QDateTime, 0x00095103,
       2, 0x80000000 | 3, 0x0009510b,

 // enums: name, alias, flags, count, data
       4,    4, 0x0,    2,   25,

 // enum data: key, value
       5, uint(KCalendarCore::Event::Opaque),
       6, uint(KCalendarCore::Event::Transparent),

       0        // eod
};

void KCalendarCore::Event::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{

#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty) {
        auto *_t = reinterpret_cast<Event *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QDateTime*>(_v) = _t->dtEnd(); break;
        case 1: *reinterpret_cast< KCalendarCore::Event::Transparency*>(_v) = _t->transparency(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = reinterpret_cast<Event *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setDtEnd(*reinterpret_cast< QDateTime*>(_v)); break;
        case 1: _t->setTransparency(*reinterpret_cast< KCalendarCore::Event::Transparency*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject KCalendarCore::Event::staticMetaObject = { {
    QtPrivate::MetaObjectForType<Incidence>::value(),
    qt_meta_stringdata_KCalendarCore__Event.data,
    qt_meta_data_KCalendarCore__Event,
    qt_static_metacall,
    nullptr,
    nullptr
} };

QT_WARNING_POP
QT_END_MOC_NAMESPACE
