/****************************************************************************
** Meta object code from reading C++ file 'yearscheduleview.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/widget/yearWidget/yearscheduleview.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'yearscheduleview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_CYearScheduleView_t {
    QByteArrayData data[1];
    char stringdata0[18];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CYearScheduleView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CYearScheduleView_t qt_meta_stringdata_CYearScheduleView = {
    {
QT_MOC_LITERAL(0, 0, 17) // "CYearScheduleView"

    },
    "CYearScheduleView"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CYearScheduleView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void CYearScheduleView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject CYearScheduleView::staticMetaObject = { {
    QMetaObject::SuperData::link<DWidget::staticMetaObject>(),
    qt_meta_stringdata_CYearScheduleView.data,
    qt_meta_data_CYearScheduleView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CYearScheduleView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CYearScheduleView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CYearScheduleView.stringdata0))
        return static_cast<void*>(this);
    return DWidget::qt_metacast(_clname);
}

int CYearScheduleView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DWidget::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_CYearScheduleOutView_t {
    QByteArrayData data[6];
    char stringdata0[80];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CYearScheduleOutView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CYearScheduleOutView_t qt_meta_stringdata_CYearScheduleOutView = {
    {
QT_MOC_LITERAL(0, 0, 20), // "CYearScheduleOutView"
QT_MOC_LITERAL(1, 21, 21), // "signalsViewSelectDate"
QT_MOC_LITERAL(2, 43, 0), // ""
QT_MOC_LITERAL(3, 44, 4), // "date"
QT_MOC_LITERAL(4, 49, 9), // "pressType"
QT_MOC_LITERAL(5, 59, 20) // "signalupdateschedule"

    },
    "CYearScheduleOutView\0signalsViewSelectDate\0"
    "\0date\0pressType\0signalupdateschedule"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CYearScheduleOutView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   29,    2, 0x06 /* Public */,
       1,    1,   34,    2, 0x26 /* Public | MethodCloned */,
       5,    0,   37,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QDate, QMetaType::Int,    3,    4,
    QMetaType::Void, QMetaType::QDate,    3,
    QMetaType::Void,

       0        // eod
};

void CYearScheduleOutView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CYearScheduleOutView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->signalsViewSelectDate((*reinterpret_cast< QDate(*)>(_a[1])),(*reinterpret_cast< const int(*)>(_a[2]))); break;
        case 1: _t->signalsViewSelectDate((*reinterpret_cast< QDate(*)>(_a[1]))); break;
        case 2: _t->signalupdateschedule(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CYearScheduleOutView::*)(QDate , const int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CYearScheduleOutView::signalsViewSelectDate)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (CYearScheduleOutView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CYearScheduleOutView::signalupdateschedule)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject CYearScheduleOutView::staticMetaObject = { {
    QMetaObject::SuperData::link<DArrowRectangle::staticMetaObject>(),
    qt_meta_stringdata_CYearScheduleOutView.data,
    qt_meta_data_CYearScheduleOutView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CYearScheduleOutView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CYearScheduleOutView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CYearScheduleOutView.stringdata0))
        return static_cast<void*>(this);
    return DArrowRectangle::qt_metacast(_clname);
}

int CYearScheduleOutView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DArrowRectangle::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void CYearScheduleOutView::signalsViewSelectDate(QDate _t1, const int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 2
void CYearScheduleOutView::signalupdateschedule()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
