/****************************************************************************
** Meta object code from reading C++ file 'calendar.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/calendar.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'calendar.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_KCalendarCore__Calendar_t {
    QByteArrayData data[16];
    char stringdata0[177];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KCalendarCore__Calendar_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KCalendarCore__Calendar_t qt_meta_stringdata_KCalendarCore__Calendar = {
    {
QT_MOC_LITERAL(0, 0, 23), // "KCalendarCore::Calendar"
QT_MOC_LITERAL(1, 24, 13), // "filterChanged"
QT_MOC_LITERAL(2, 38, 0), // ""
QT_MOC_LITERAL(3, 39, 9), // "idChanged"
QT_MOC_LITERAL(4, 49, 11), // "nameChanged"
QT_MOC_LITERAL(5, 61, 11), // "iconChanged"
QT_MOC_LITERAL(6, 73, 17), // "accessModeChanged"
QT_MOC_LITERAL(7, 91, 12), // "ownerChanged"
QT_MOC_LITERAL(8, 104, 9), // "productId"
QT_MOC_LITERAL(9, 114, 5), // "owner"
QT_MOC_LITERAL(10, 120, 21), // "KCalendarCore::Person"
QT_MOC_LITERAL(11, 142, 2), // "id"
QT_MOC_LITERAL(12, 145, 4), // "name"
QT_MOC_LITERAL(13, 150, 4), // "icon"
QT_MOC_LITERAL(14, 155, 10), // "accessMode"
QT_MOC_LITERAL(15, 166, 10) // "AccessMode"

    },
    "KCalendarCore::Calendar\0filterChanged\0"
    "\0idChanged\0nameChanged\0iconChanged\0"
    "accessModeChanged\0ownerChanged\0productId\0"
    "owner\0KCalendarCore::Person\0id\0name\0"
    "icon\0accessMode\0AccessMode"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KCalendarCore__Calendar[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       6,   50, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       6,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   44,    2, 0x06 /* Public */,
       3,    0,   45,    2, 0x06 /* Public */,
       4,    0,   46,    2, 0x06 /* Public */,
       5,    0,   47,    2, 0x06 /* Public */,
       6,    0,   48,    2, 0x06 /* Public */,
       7,    0,   49,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
       8, QMetaType::QString, 0x00095103,
       9, 0x80000000 | 10, 0x0049510b,
      11, QMetaType::QString, 0x00495103,
      12, QMetaType::QString, 0x00495103,
      13, QMetaType::QIcon, 0x00495103,
      14, 0x80000000 | 15, 0x0049510b,

 // properties: notify_signal_id
       0,
       5,
       1,
       2,
       3,
       4,

       0        // eod
};

void KCalendarCore::Calendar::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Calendar *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->filterChanged(); break;
        case 1: _t->idChanged(); break;
        case 2: _t->nameChanged(); break;
        case 3: _t->iconChanged(); break;
        case 4: _t->accessModeChanged(); break;
        case 5: _t->ownerChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (Calendar::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Calendar::filterChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (Calendar::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Calendar::idChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (Calendar::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Calendar::nameChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (Calendar::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Calendar::iconChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (Calendar::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Calendar::accessModeChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (Calendar::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Calendar::ownerChanged)) {
                *result = 5;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KCalendarCore::Person >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<Calendar *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QString*>(_v) = _t->productId(); break;
        case 1: *reinterpret_cast< KCalendarCore::Person*>(_v) = _t->owner(); break;
        case 2: *reinterpret_cast< QString*>(_v) = _t->id(); break;
        case 3: *reinterpret_cast< QString*>(_v) = _t->name(); break;
        case 4: *reinterpret_cast< QIcon*>(_v) = _t->icon(); break;
        case 5: *reinterpret_cast< AccessMode*>(_v) = _t->accessMode(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<Calendar *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setProductId(*reinterpret_cast< QString*>(_v)); break;
        case 1: _t->setOwner(*reinterpret_cast< KCalendarCore::Person*>(_v)); break;
        case 2: _t->setId(*reinterpret_cast< QString*>(_v)); break;
        case 3: _t->setName(*reinterpret_cast< QString*>(_v)); break;
        case 4: _t->setIcon(*reinterpret_cast< QIcon*>(_v)); break;
        case 5: _t->setAccessMode(*reinterpret_cast< AccessMode*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject KCalendarCore::Calendar::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_KCalendarCore__Calendar.data,
    qt_meta_data_KCalendarCore__Calendar,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *KCalendarCore::Calendar::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KCalendarCore::Calendar::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_KCalendarCore__Calendar.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "CustomProperties"))
        return static_cast< CustomProperties*>(this);
    if (!strcmp(_clname, "IncidenceBase::IncidenceObserver"))
        return static_cast< IncidenceBase::IncidenceObserver*>(this);
    return QObject::qt_metacast(_clname);
}

int KCalendarCore::Calendar::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 6;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void KCalendarCore::Calendar::filterChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void KCalendarCore::Calendar::idChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void KCalendarCore::Calendar::nameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void KCalendarCore::Calendar::iconChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void KCalendarCore::Calendar::accessModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void KCalendarCore::Calendar::ownerChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
