/****************************************************************************
** Meta object code from reading C++ file 'weekview.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/widget/weekWidget/weekview.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'weekview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_CWeekView_t {
    QByteArrayData data[9];
    char stringdata0[96];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CWeekView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CWeekView_t qt_meta_stringdata_CWeekView = {
    {
QT_MOC_LITERAL(0, 0, 9), // "CWeekView"
QT_MOC_LITERAL(1, 10, 17), // "signalsSelectDate"
QT_MOC_LITERAL(2, 28, 0), // ""
QT_MOC_LITERAL(3, 29, 4), // "date"
QT_MOC_LITERAL(4, 34, 16), // "signalIsDragging"
QT_MOC_LITERAL(5, 51, 5), // "bool&"
QT_MOC_LITERAL(6, 57, 10), // "isDragging"
QT_MOC_LITERAL(7, 68, 13), // "signalBtnPrev"
QT_MOC_LITERAL(8, 82, 13) // "signalBtnNext"

    },
    "CWeekView\0signalsSelectDate\0\0date\0"
    "signalIsDragging\0bool&\0isDragging\0"
    "signalBtnPrev\0signalBtnNext"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CWeekView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       4,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   34,    2, 0x06 /* Public */,
       4,    1,   37,    2, 0x06 /* Public */,
       7,    0,   40,    2, 0x06 /* Public */,
       8,    0,   41,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QDate,    3,
    QMetaType::Void, 0x80000000 | 5,    6,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void CWeekView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CWeekView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->signalsSelectDate((*reinterpret_cast< const QDate(*)>(_a[1]))); break;
        case 1: _t->signalIsDragging((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 2: _t->signalBtnPrev(); break;
        case 3: _t->signalBtnNext(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CWeekView::*)(const QDate & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CWeekView::signalsSelectDate)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (CWeekView::*)(bool & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CWeekView::signalIsDragging)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (CWeekView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CWeekView::signalBtnPrev)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (CWeekView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CWeekView::signalBtnNext)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject CWeekView::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_CWeekView.data,
    qt_meta_data_CWeekView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CWeekView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CWeekView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CWeekView.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int CWeekView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void CWeekView::signalsSelectDate(const QDate & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void CWeekView::signalIsDragging(bool & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void CWeekView::signalBtnPrev()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void CWeekView::signalBtnNext()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
struct qt_meta_stringdata_CWeekNumWidget_t {
    QByteArrayData data[9];
    char stringdata0[93];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CWeekNumWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CWeekNumWidget_t qt_meta_stringdata_CWeekNumWidget = {
    {
QT_MOC_LITERAL(0, 0, 14), // "CWeekNumWidget"
QT_MOC_LITERAL(1, 15, 17), // "signalsSelectDate"
QT_MOC_LITERAL(2, 33, 0), // ""
QT_MOC_LITERAL(3, 34, 4), // "date"
QT_MOC_LITERAL(4, 39, 13), // "signalBtnPrev"
QT_MOC_LITERAL(5, 53, 13), // "signalBtnNext"
QT_MOC_LITERAL(6, 67, 11), // "cellClicked"
QT_MOC_LITERAL(7, 79, 8), // "QWidget*"
QT_MOC_LITERAL(8, 88, 4) // "cell"

    },
    "CWeekNumWidget\0signalsSelectDate\0\0"
    "date\0signalBtnPrev\0signalBtnNext\0"
    "cellClicked\0QWidget*\0cell"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CWeekNumWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       4,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   34,    2, 0x06 /* Public */,
       4,    0,   37,    2, 0x06 /* Public */,
       5,    0,   38,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    1,   39,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QDate,    3,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 7,    8,

       0        // eod
};

void CWeekNumWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CWeekNumWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->signalsSelectDate((*reinterpret_cast< const QDate(*)>(_a[1]))); break;
        case 1: _t->signalBtnPrev(); break;
        case 2: _t->signalBtnNext(); break;
        case 3: _t->cellClicked((*reinterpret_cast< QWidget*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWidget* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CWeekNumWidget::*)(const QDate & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CWeekNumWidget::signalsSelectDate)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (CWeekNumWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CWeekNumWidget::signalBtnPrev)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (CWeekNumWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CWeekNumWidget::signalBtnNext)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject CWeekNumWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_CWeekNumWidget.data,
    qt_meta_data_CWeekNumWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CWeekNumWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CWeekNumWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CWeekNumWidget.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int CWeekNumWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void CWeekNumWidget::signalsSelectDate(const QDate & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void CWeekNumWidget::signalBtnPrev()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void CWeekNumWidget::signalBtnNext()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
