/****************************************************************************
** Meta object code from reading C++ file 'incidencebase.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/incidencebase.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'incidencebase.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_KCalendarCore__IncidenceBase_t {
    QByteArrayData data[8];
    char stringdata0[103];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KCalendarCore__IncidenceBase_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KCalendarCore__IncidenceBase_t qt_meta_stringdata_KCalendarCore__IncidenceBase = {
    {
QT_MOC_LITERAL(0, 0, 28), // "KCalendarCore::IncidenceBase"
QT_MOC_LITERAL(1, 29, 3), // "uid"
QT_MOC_LITERAL(2, 33, 12), // "lastModified"
QT_MOC_LITERAL(3, 46, 7), // "dtStart"
QT_MOC_LITERAL(4, 54, 6), // "allDay"
QT_MOC_LITERAL(5, 61, 9), // "organizer"
QT_MOC_LITERAL(6, 71, 21), // "KCalendarCore::Person"
QT_MOC_LITERAL(7, 93, 9) // "attendees"

    },
    "KCalendarCore::IncidenceBase\0uid\0"
    "lastModified\0dtStart\0allDay\0organizer\0"
    "KCalendarCore::Person\0attendees"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KCalendarCore__IncidenceBase[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       6,   14, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       4,       // flags
       0,       // signalCount

 // properties: name, type, flags
       1, QMetaType::QString, 0x00095103,
       2, QMetaType::QDateTime, 0x00095103,
       3, QMetaType::QDateTime, 0x00095103,
       4, QMetaType::Bool, 0x00095103,
       5, 0x80000000 | 6, 0x0009510b,
       7, QMetaType::QVariantList, 0x00095001,

       0        // eod
};

void KCalendarCore::IncidenceBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KCalendarCore::Person >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = reinterpret_cast<IncidenceBase *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QString*>(_v) = _t->uid(); break;
        case 1: *reinterpret_cast< QDateTime*>(_v) = _t->lastModified(); break;
        case 2: *reinterpret_cast< QDateTime*>(_v) = _t->dtStart(); break;
        case 3: *reinterpret_cast< bool*>(_v) = _t->allDay(); break;
        case 4: *reinterpret_cast< KCalendarCore::Person*>(_v) = _t->organizer(); break;
        case 5: *reinterpret_cast< QVariantList*>(_v) = _t->attendeesVariant(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = reinterpret_cast<IncidenceBase *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setUid(*reinterpret_cast< QString*>(_v)); break;
        case 1: _t->setLastModified(*reinterpret_cast< QDateTime*>(_v)); break;
        case 2: _t->setDtStart(*reinterpret_cast< QDateTime*>(_v)); break;
        case 3: _t->setAllDay(*reinterpret_cast< bool*>(_v)); break;
        case 4: _t->setOrganizer(*reinterpret_cast< KCalendarCore::Person*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_o;
}

QT_INIT_METAOBJECT const QMetaObject KCalendarCore::IncidenceBase::staticMetaObject = { {
    QtPrivate::MetaObjectForType<CustomProperties>::value(),
    qt_meta_stringdata_KCalendarCore__IncidenceBase.data,
    qt_meta_data_KCalendarCore__IncidenceBase,
    qt_static_metacall,
    nullptr,
    nullptr
} };

QT_WARNING_POP
QT_END_MOC_NAMESPACE
