/****************************************************************************
** Meta object code from reading C++ file 'schedulesearchview.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/widget/schedulesearchview.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'schedulesearchview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_CScheduleSearchView_t {
    QByteArrayData data[18];
    char stringdata0[294];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CScheduleSearchView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CScheduleSearchView_t qt_meta_stringdata_CScheduleSearchView = {
    {
QT_MOC_LITERAL(0, 0, 19), // "CScheduleSearchView"
QT_MOC_LITERAL(1, 20, 20), // "signalSelectSchedule"
QT_MOC_LITERAL(2, 41, 0), // ""
QT_MOC_LITERAL(3, 42, 14), // "DSchedule::Ptr"
QT_MOC_LITERAL(4, 57, 12), // "scheduleInfo"
QT_MOC_LITERAL(5, 70, 18), // "signalScheduleHide"
QT_MOC_LITERAL(6, 89, 23), // "signalSelectCurrentItem"
QT_MOC_LITERAL(7, 113, 21), // "signalSchotCutClicked"
QT_MOC_LITERAL(8, 135, 13), // "slotsetSearch"
QT_MOC_LITERAL(9, 149, 3), // "str"
QT_MOC_LITERAL(10, 153, 18), // "slotSelectSchedule"
QT_MOC_LITERAL(11, 172, 12), // "updateSearch"
QT_MOC_LITERAL(12, 185, 21), // "slotSelectCurrentItem"
QT_MOC_LITERAL(13, 207, 20), // "CScheduleSearchItem*"
QT_MOC_LITERAL(14, 228, 4), // "item"
QT_MOC_LITERAL(15, 233, 12), // "itemFocusOut"
QT_MOC_LITERAL(16, 246, 21), // "slotListWidgetClicked"
QT_MOC_LITERAL(17, 268, 25) // "slotScearedScheduleUpdate"

    },
    "CScheduleSearchView\0signalSelectSchedule\0"
    "\0DSchedule::Ptr\0scheduleInfo\0"
    "signalScheduleHide\0signalSelectCurrentItem\0"
    "signalSchotCutClicked\0slotsetSearch\0"
    "str\0slotSelectSchedule\0updateSearch\0"
    "slotSelectCurrentItem\0CScheduleSearchItem*\0"
    "item\0itemFocusOut\0slotListWidgetClicked\0"
    "slotScearedScheduleUpdate"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CScheduleSearchView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   64,    2, 0x06 /* Public */,
       5,    0,   67,    2, 0x06 /* Public */,
       6,    0,   68,    2, 0x06 /* Public */,
       7,    0,   69,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       8,    1,   70,    2, 0x0a /* Public */,
      10,    1,   73,    2, 0x0a /* Public */,
      11,    0,   76,    2, 0x0a /* Public */,
      12,    2,   77,    2, 0x0a /* Public */,
      16,    0,   82,    2, 0x0a /* Public */,
      17,    0,   83,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,    9,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 13, QMetaType::Bool,   14,   15,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void CScheduleSearchView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CScheduleSearchView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->signalSelectSchedule((*reinterpret_cast< const DSchedule::Ptr(*)>(_a[1]))); break;
        case 1: _t->signalScheduleHide(); break;
        case 2: _t->signalSelectCurrentItem(); break;
        case 3: _t->signalSchotCutClicked(); break;
        case 4: _t->slotsetSearch((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 5: _t->slotSelectSchedule((*reinterpret_cast< const DSchedule::Ptr(*)>(_a[1]))); break;
        case 6: _t->updateSearch(); break;
        case 7: _t->slotSelectCurrentItem((*reinterpret_cast< CScheduleSearchItem*(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 8: _t->slotListWidgetClicked(); break;
        case 9: _t->slotScearedScheduleUpdate(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< CScheduleSearchItem* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CScheduleSearchView::*)(const DSchedule::Ptr & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CScheduleSearchView::signalSelectSchedule)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (CScheduleSearchView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CScheduleSearchView::signalScheduleHide)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (CScheduleSearchView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CScheduleSearchView::signalSelectCurrentItem)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (CScheduleSearchView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CScheduleSearchView::signalSchotCutClicked)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject CScheduleSearchView::staticMetaObject = { {
    QMetaObject::SuperData::link<DWidget::staticMetaObject>(),
    qt_meta_stringdata_CScheduleSearchView.data,
    qt_meta_data_CScheduleSearchView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CScheduleSearchView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CScheduleSearchView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CScheduleSearchView.stringdata0))
        return static_cast<void*>(this);
    return DWidget::qt_metacast(_clname);
}

int CScheduleSearchView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void CScheduleSearchView::signalSelectSchedule(const DSchedule::Ptr & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void CScheduleSearchView::signalScheduleHide()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void CScheduleSearchView::signalSelectCurrentItem()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void CScheduleSearchView::signalSchotCutClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
struct qt_meta_stringdata_CScheduleListWidget_t {
    QByteArrayData data[3];
    char stringdata0[45];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CScheduleListWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CScheduleListWidget_t qt_meta_stringdata_CScheduleListWidget = {
    {
QT_MOC_LITERAL(0, 0, 19), // "CScheduleListWidget"
QT_MOC_LITERAL(1, 20, 23), // "signalListWidgetClicked"
QT_MOC_LITERAL(2, 44, 0) // ""

    },
    "CScheduleListWidget\0signalListWidgetClicked\0"
    ""
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CScheduleListWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,

       0        // eod
};

void CScheduleListWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CScheduleListWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->signalListWidgetClicked(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CScheduleListWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CScheduleListWidget::signalListWidgetClicked)) {
                *result = 0;
                return;
            }
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject CScheduleListWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<DListWidget::staticMetaObject>(),
    qt_meta_stringdata_CScheduleListWidget.data,
    qt_meta_data_CScheduleListWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CScheduleListWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CScheduleListWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CScheduleListWidget.stringdata0))
        return static_cast<void*>(this);
    return DListWidget::qt_metacast(_clname);
}

int CScheduleListWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DListWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void CScheduleListWidget::signalListWidgetClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
struct qt_meta_stringdata_CScheduleSearchItem_t {
    QByteArrayData data[16];
    char stringdata0[229];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CScheduleSearchItem_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CScheduleSearchItem_t qt_meta_stringdata_CScheduleSearchItem = {
    {
QT_MOC_LITERAL(0, 0, 19), // "CScheduleSearchItem"
QT_MOC_LITERAL(1, 20, 13), // "signalsDelete"
QT_MOC_LITERAL(2, 34, 0), // ""
QT_MOC_LITERAL(3, 35, 20), // "CScheduleSearchItem*"
QT_MOC_LITERAL(4, 56, 4), // "item"
QT_MOC_LITERAL(5, 61, 11), // "signalsEdit"
QT_MOC_LITERAL(6, 73, 20), // "signalSelectSchedule"
QT_MOC_LITERAL(7, 94, 14), // "DSchedule::Ptr"
QT_MOC_LITERAL(8, 109, 12), // "scheduleInfo"
QT_MOC_LITERAL(9, 122, 23), // "signalSelectCurrentItem"
QT_MOC_LITERAL(10, 146, 14), // "focusOutStatus"
QT_MOC_LITERAL(11, 161, 8), // "slotEdit"
QT_MOC_LITERAL(12, 170, 10), // "slotDelete"
QT_MOC_LITERAL(13, 181, 21), // "slotTimeFormatChanged"
QT_MOC_LITERAL(14, 203, 5), // "value"
QT_MOC_LITERAL(15, 209, 19) // "slotSchotCutClicked"

    },
    "CScheduleSearchItem\0signalsDelete\0\0"
    "CScheduleSearchItem*\0item\0signalsEdit\0"
    "signalSelectSchedule\0DSchedule::Ptr\0"
    "scheduleInfo\0signalSelectCurrentItem\0"
    "focusOutStatus\0slotEdit\0slotDelete\0"
    "slotTimeFormatChanged\0value\0"
    "slotSchotCutClicked"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CScheduleSearchItem[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   54,    2, 0x06 /* Public */,
       5,    1,   57,    2, 0x06 /* Public */,
       6,    1,   60,    2, 0x06 /* Public */,
       9,    2,   63,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      11,    0,   68,    2, 0x0a /* Public */,
      12,    0,   69,    2, 0x0a /* Public */,
      13,    1,   70,    2, 0x0a /* Public */,
      15,    0,   73,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Bool,    4,   10,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   14,
    QMetaType::Void,

       0        // eod
};

void CScheduleSearchItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CScheduleSearchItem *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->signalsDelete((*reinterpret_cast< CScheduleSearchItem*(*)>(_a[1]))); break;
        case 1: _t->signalsEdit((*reinterpret_cast< CScheduleSearchItem*(*)>(_a[1]))); break;
        case 2: _t->signalSelectSchedule((*reinterpret_cast< const DSchedule::Ptr(*)>(_a[1]))); break;
        case 3: _t->signalSelectCurrentItem((*reinterpret_cast< CScheduleSearchItem*(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 4: _t->slotEdit(); break;
        case 5: _t->slotDelete(); break;
        case 6: _t->slotTimeFormatChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->slotSchotCutClicked(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< CScheduleSearchItem* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< CScheduleSearchItem* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< CScheduleSearchItem* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CScheduleSearchItem::*)(CScheduleSearchItem * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CScheduleSearchItem::signalsDelete)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (CScheduleSearchItem::*)(CScheduleSearchItem * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CScheduleSearchItem::signalsEdit)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (CScheduleSearchItem::*)(const DSchedule::Ptr & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CScheduleSearchItem::signalSelectSchedule)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (CScheduleSearchItem::*)(CScheduleSearchItem * , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CScheduleSearchItem::signalSelectCurrentItem)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject CScheduleSearchItem::staticMetaObject = { {
    QMetaObject::SuperData::link<DLabel::staticMetaObject>(),
    qt_meta_stringdata_CScheduleSearchItem.data,
    qt_meta_data_CScheduleSearchItem,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CScheduleSearchItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CScheduleSearchItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CScheduleSearchItem.stringdata0))
        return static_cast<void*>(this);
    return DLabel::qt_metacast(_clname);
}

int CScheduleSearchItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DLabel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void CScheduleSearchItem::signalsDelete(CScheduleSearchItem * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void CScheduleSearchItem::signalsEdit(CScheduleSearchItem * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void CScheduleSearchItem::signalSelectSchedule(const DSchedule::Ptr & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void CScheduleSearchItem::signalSelectCurrentItem(CScheduleSearchItem * _t1, bool _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}
struct qt_meta_stringdata_CScheduleSearchDateItem_t {
    QByteArrayData data[3];
    char stringdata0[49];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CScheduleSearchDateItem_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CScheduleSearchDateItem_t qt_meta_stringdata_CScheduleSearchDateItem = {
    {
QT_MOC_LITERAL(0, 0, 23), // "CScheduleSearchDateItem"
QT_MOC_LITERAL(1, 24, 23), // "signalLabelScheduleHide"
QT_MOC_LITERAL(2, 48, 0) // ""

    },
    "CScheduleSearchDateItem\0signalLabelScheduleHide\0"
    ""
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CScheduleSearchDateItem[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,

       0        // eod
};

void CScheduleSearchDateItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CScheduleSearchDateItem *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->signalLabelScheduleHide(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CScheduleSearchDateItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CScheduleSearchDateItem::signalLabelScheduleHide)) {
                *result = 0;
                return;
            }
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject CScheduleSearchDateItem::staticMetaObject = { {
    QMetaObject::SuperData::link<DLabel::staticMetaObject>(),
    qt_meta_stringdata_CScheduleSearchDateItem.data,
    qt_meta_data_CScheduleSearchDateItem,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CScheduleSearchDateItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CScheduleSearchDateItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CScheduleSearchDateItem.stringdata0))
        return static_cast<void*>(this);
    return DLabel::qt_metacast(_clname);
}

int CScheduleSearchDateItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DLabel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void CScheduleSearchDateItem::signalLabelScheduleHide()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
