/****************************************************************************
** Meta object code from reading C++ file 'weekwindow.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/widget/weekWidget/weekwindow.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'weekwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_CWeekWindow_t {
    QByteArrayData data[18];
    char stringdata0[227];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CWeekWindow_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CWeekWindow_t qt_meta_stringdata_CWeekWindow = {
    {
QT_MOC_LITERAL(0, 0, 11), // "CWeekWindow"
QT_MOC_LITERAL(1, 12, 17), // "signalCurrentDate"
QT_MOC_LITERAL(2, 30, 0), // ""
QT_MOC_LITERAL(3, 31, 4), // "date"
QT_MOC_LITERAL(4, 36, 21), // "signalsViewSelectDate"
QT_MOC_LITERAL(5, 58, 14), // "slotIsDragging"
QT_MOC_LITERAL(6, 73, 5), // "bool&"
QT_MOC_LITERAL(7, 79, 10), // "isDragging"
QT_MOC_LITERAL(8, 90, 18), // "slotViewSelectDate"
QT_MOC_LITERAL(9, 109, 17), // "slotSwitchPrePage"
QT_MOC_LITERAL(10, 127, 18), // "slotSwitchNextPage"
QT_MOC_LITERAL(11, 146, 16), // "slotScheduleHide"
QT_MOC_LITERAL(12, 163, 8), // "slotprev"
QT_MOC_LITERAL(13, 172, 8), // "slotnext"
QT_MOC_LITERAL(14, 181, 9), // "slottoday"
QT_MOC_LITERAL(15, 191, 14), // "slotSelectDate"
QT_MOC_LITERAL(16, 206, 14), // "slotAngleDelta"
QT_MOC_LITERAL(17, 221, 5) // "delta"

    },
    "CWeekWindow\0signalCurrentDate\0\0date\0"
    "signalsViewSelectDate\0slotIsDragging\0"
    "bool&\0isDragging\0slotViewSelectDate\0"
    "slotSwitchPrePage\0slotSwitchNextPage\0"
    "slotScheduleHide\0slotprev\0slotnext\0"
    "slottoday\0slotSelectDate\0slotAngleDelta\0"
    "delta"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CWeekWindow[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   74,    2, 0x06 /* Public */,
       4,    1,   77,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    1,   80,    2, 0x0a /* Public */,
       8,    1,   83,    2, 0x0a /* Public */,
       9,    0,   86,    2, 0x0a /* Public */,
      10,    0,   87,    2, 0x0a /* Public */,
      11,    0,   88,    2, 0x0a /* Public */,
      12,    0,   89,    2, 0x08 /* Private */,
      13,    0,   90,    2, 0x08 /* Private */,
      14,    0,   91,    2, 0x08 /* Private */,
      15,    1,   92,    2, 0x08 /* Private */,
      16,    1,   95,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QDate,    3,
    QMetaType::Void, QMetaType::QDate,    3,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void, QMetaType::QDate,    3,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QDate,    3,
    QMetaType::Void, QMetaType::Int,   17,

       0        // eod
};

void CWeekWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CWeekWindow *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->signalCurrentDate((*reinterpret_cast< QDate(*)>(_a[1]))); break;
        case 1: _t->signalsViewSelectDate((*reinterpret_cast< QDate(*)>(_a[1]))); break;
        case 2: _t->slotIsDragging((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->slotViewSelectDate((*reinterpret_cast< const QDate(*)>(_a[1]))); break;
        case 4: _t->slotSwitchPrePage(); break;
        case 5: _t->slotSwitchNextPage(); break;
        case 6: _t->slotScheduleHide(); break;
        case 7: _t->slotprev(); break;
        case 8: _t->slotnext(); break;
        case 9: _t->slottoday(); break;
        case 10: _t->slotSelectDate((*reinterpret_cast< const QDate(*)>(_a[1]))); break;
        case 11: _t->slotAngleDelta((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CWeekWindow::*)(QDate );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CWeekWindow::signalCurrentDate)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (CWeekWindow::*)(QDate );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CWeekWindow::signalsViewSelectDate)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject CWeekWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<CScheduleBaseWidget::staticMetaObject>(),
    qt_meta_stringdata_CWeekWindow.data,
    qt_meta_data_CWeekWindow,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CWeekWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CWeekWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CWeekWindow.stringdata0))
        return static_cast<void*>(this);
    return CScheduleBaseWidget::qt_metacast(_clname);
}

int CWeekWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = CScheduleBaseWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void CWeekWindow::signalCurrentDate(QDate _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void CWeekWindow::signalsViewSelectDate(QDate _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
