/****************************************************************************
** Meta object code from reading C++ file 'calendarmainwindow.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/widget/calendarmainwindow.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'calendarmainwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Calendarmainwindow_t {
    QByteArrayData data[40];
    char stringdata0[625];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Calendarmainwindow_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Calendarmainwindow_t qt_meta_stringdata_Calendarmainwindow = {
    {
QT_MOC_LITERAL(0, 0, 18), // "Calendarmainwindow"
QT_MOC_LITERAL(1, 19, 9), // "slotTheme"
QT_MOC_LITERAL(2, 29, 0), // ""
QT_MOC_LITERAL(3, 30, 4), // "type"
QT_MOC_LITERAL(4, 35, 16), // "slotOpenSchedule"
QT_MOC_LITERAL(5, 52, 3), // "job"
QT_MOC_LITERAL(6, 56, 16), // "slotViewShortcut"
QT_MOC_LITERAL(7, 73, 21), // "slotCurrentDateUpdate"
QT_MOC_LITERAL(8, 95, 18), // "slotSetSearchFocus"
QT_MOC_LITERAL(9, 114, 27), // "slotapplicationStateChanged"
QT_MOC_LITERAL(10, 142, 20), // "Qt::ApplicationState"
QT_MOC_LITERAL(11, 163, 5), // "state"
QT_MOC_LITERAL(12, 169, 15), // "setScheduleHide"
QT_MOC_LITERAL(13, 185, 17), // "slotstackWClicked"
QT_MOC_LITERAL(14, 203, 16), // "QAbstractButton*"
QT_MOC_LITERAL(15, 220, 2), // "bt"
QT_MOC_LITERAL(16, 223, 19), // "slotWUpdateSchedule"
QT_MOC_LITERAL(17, 243, 18), // "slotSreturnPressed"
QT_MOC_LITERAL(18, 262, 16), // "slotStextChanged"
QT_MOC_LITERAL(19, 279, 21), // "slotStextfocusChanged"
QT_MOC_LITERAL(20, 301, 7), // "onFocus"
QT_MOC_LITERAL(21, 309, 14), // "slotSearchEdit"
QT_MOC_LITERAL(22, 324, 24), // "slotSearchSelectSchedule"
QT_MOC_LITERAL(23, 349, 14), // "DSchedule::Ptr"
QT_MOC_LITERAL(24, 364, 12), // "scheduleInfo"
QT_MOC_LITERAL(25, 377, 24), // "slotViewtransparentFrame"
QT_MOC_LITERAL(26, 402, 6), // "isShow"
QT_MOC_LITERAL(27, 409, 16), // "slotSetButtonBox"
QT_MOC_LITERAL(28, 426, 14), // "slotSwitchView"
QT_MOC_LITERAL(29, 441, 9), // "viewIndex"
QT_MOC_LITERAL(30, 451, 15), // "slotNewSchedule"
QT_MOC_LITERAL(31, 467, 21), // "slotOpenSettingDialog"
QT_MOC_LITERAL(32, 489, 15), // "slotShowPrivacy"
QT_MOC_LITERAL(33, 505, 14), // "slotDeleteitem"
QT_MOC_LITERAL(34, 520, 14), // "slotSetMaxSize"
QT_MOC_LITERAL(35, 535, 21), // "slotSearchFocusSwitch"
QT_MOC_LITERAL(36, 557, 23), // "slotSidebarStatusChange"
QT_MOC_LITERAL(37, 581, 17), // "slotShowSyncToast"
QT_MOC_LITERAL(38, 599, 7), // "syncNum"
QT_MOC_LITERAL(39, 607, 17) // "slotAccountUpdate"

    },
    "Calendarmainwindow\0slotTheme\0\0type\0"
    "slotOpenSchedule\0job\0slotViewShortcut\0"
    "slotCurrentDateUpdate\0slotSetSearchFocus\0"
    "slotapplicationStateChanged\0"
    "Qt::ApplicationState\0state\0setScheduleHide\0"
    "slotstackWClicked\0QAbstractButton*\0"
    "bt\0slotWUpdateSchedule\0slotSreturnPressed\0"
    "slotStextChanged\0slotStextfocusChanged\0"
    "onFocus\0slotSearchEdit\0slotSearchSelectSchedule\0"
    "DSchedule::Ptr\0scheduleInfo\0"
    "slotViewtransparentFrame\0isShow\0"
    "slotSetButtonBox\0slotSwitchView\0"
    "viewIndex\0slotNewSchedule\0"
    "slotOpenSettingDialog\0slotShowPrivacy\0"
    "slotDeleteitem\0slotSetMaxSize\0"
    "slotSearchFocusSwitch\0slotSidebarStatusChange\0"
    "slotShowSyncToast\0syncNum\0slotAccountUpdate"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Calendarmainwindow[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      26,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,  144,    2, 0x0a /* Public */,
       4,    1,  147,    2, 0x0a /* Public */,
       6,    0,  150,    2, 0x0a /* Public */,
       7,    0,  151,    2, 0x0a /* Public */,
       8,    0,  152,    2, 0x0a /* Public */,
       9,    1,  153,    2, 0x08 /* Private */,
      12,    0,  156,    2, 0x08 /* Private */,
      13,    1,  157,    2, 0x08 /* Private */,
      16,    0,  160,    2, 0x08 /* Private */,
      17,    0,  161,    2, 0x08 /* Private */,
      18,    0,  162,    2, 0x08 /* Private */,
      19,    1,  163,    2, 0x08 /* Private */,
      21,    0,  166,    2, 0x08 /* Private */,
      22,    1,  167,    2, 0x08 /* Private */,
      25,    1,  170,    2, 0x08 /* Private */,
      27,    0,  173,    2, 0x08 /* Private */,
      28,    1,  174,    2, 0x08 /* Private */,
      30,    0,  177,    2, 0x08 /* Private */,
      31,    0,  178,    2, 0x08 /* Private */,
      32,    0,  179,    2, 0x08 /* Private */,
      33,    0,  180,    2, 0x08 /* Private */,
      34,    0,  181,    2, 0x08 /* Private */,
      35,    0,  182,    2, 0x08 /* Private */,
      36,    1,  183,    2, 0x08 /* Private */,
      37,    1,  186,    2, 0x08 /* Private */,
      39,    0,  189,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 10,   11,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 14,   15,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   20,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 23,   24,
    QMetaType::Void, QMetaType::Bool,   26,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   29,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Int,   38,
    QMetaType::Void,

       0        // eod
};

void Calendarmainwindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Calendarmainwindow *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->slotTheme((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->slotOpenSchedule((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 2: _t->slotViewShortcut(); break;
        case 3: _t->slotCurrentDateUpdate(); break;
        case 4: _t->slotSetSearchFocus(); break;
        case 5: _t->slotapplicationStateChanged((*reinterpret_cast< Qt::ApplicationState(*)>(_a[1]))); break;
        case 6: _t->setScheduleHide(); break;
        case 7: _t->slotstackWClicked((*reinterpret_cast< QAbstractButton*(*)>(_a[1]))); break;
        case 8: _t->slotWUpdateSchedule(); break;
        case 9: _t->slotSreturnPressed(); break;
        case 10: _t->slotStextChanged(); break;
        case 11: _t->slotStextfocusChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 12: _t->slotSearchEdit(); break;
        case 13: _t->slotSearchSelectSchedule((*reinterpret_cast< const DSchedule::Ptr(*)>(_a[1]))); break;
        case 14: _t->slotViewtransparentFrame((*reinterpret_cast< const bool(*)>(_a[1]))); break;
        case 15: _t->slotSetButtonBox(); break;
        case 16: _t->slotSwitchView((*reinterpret_cast< const int(*)>(_a[1]))); break;
        case 17: _t->slotNewSchedule(); break;
        case 18: _t->slotOpenSettingDialog(); break;
        case 19: _t->slotShowPrivacy(); break;
        case 20: _t->slotDeleteitem(); break;
        case 21: _t->slotSetMaxSize(); break;
        case 22: _t->slotSearchFocusSwitch(); break;
        case 23: _t->slotSidebarStatusChange((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 24: _t->slotShowSyncToast((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 25: _t->slotAccountUpdate(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstractButton* >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Calendarmainwindow::staticMetaObject = { {
    QMetaObject::SuperData::link<DMainWindow::staticMetaObject>(),
    qt_meta_stringdata_Calendarmainwindow.data,
    qt_meta_data_Calendarmainwindow,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Calendarmainwindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Calendarmainwindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Calendarmainwindow.stringdata0))
        return static_cast<void*>(this);
    return DMainWindow::qt_metacast(_clname);
}

int Calendarmainwindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 26)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 26;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 26)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 26;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
