/****************************************************************************
** Meta object code from reading C++ file 'settingdialog.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/dialog/settingdialog.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'settingdialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_CSettingDialog_t {
    QByteArrayData data[23];
    char stringdata0[433];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CSettingDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CSettingDialog_t qt_meta_stringdata_CSettingDialog = {
    {
QT_MOC_LITERAL(0, 0, 14), // "CSettingDialog"
QT_MOC_LITERAL(1, 15, 25), // "slotGeneralSettingsUpdate"
QT_MOC_LITERAL(2, 41, 0), // ""
QT_MOC_LITERAL(3, 42, 17), // "slotAccountUpdate"
QT_MOC_LITERAL(4, 60, 10), // "slotLogout"
QT_MOC_LITERAL(5, 71, 14), // "DAccount::Type"
QT_MOC_LITERAL(6, 86, 22), // "slotLastSyncTimeUpdate"
QT_MOC_LITERAL(7, 109, 8), // "datetime"
QT_MOC_LITERAL(8, 118, 22), // "slotAccountStateChange"
QT_MOC_LITERAL(9, 141, 32), // "slotFirstDayofWeekCurrentChanged"
QT_MOC_LITERAL(10, 174, 5), // "index"
QT_MOC_LITERAL(11, 180, 26), // "slotTimeTypeCurrentChanged"
QT_MOC_LITERAL(12, 207, 25), // "slotAccountCurrentChanged"
QT_MOC_LITERAL(13, 233, 22), // "slotTypeAddBtnClickded"
QT_MOC_LITERAL(14, 256, 25), // "slotTypeImportBtnClickded"
QT_MOC_LITERAL(15, 282, 18), // "slotSetUosSyncFreq"
QT_MOC_LITERAL(16, 301, 4), // "freq"
QT_MOC_LITERAL(17, 306, 17), // "slotUosManualSync"
QT_MOC_LITERAL(18, 324, 22), // "slotNetworkStateChange"
QT_MOC_LITERAL(19, 347, 28), // "DOANetWorkDBus::NetWorkState"
QT_MOC_LITERAL(20, 376, 5), // "state"
QT_MOC_LITERAL(21, 382, 23), // "slotSyncTagButtonUpdate"
QT_MOC_LITERAL(22, 406, 26) // "slotSyncAccountStateUpdate"

    },
    "CSettingDialog\0slotGeneralSettingsUpdate\0"
    "\0slotAccountUpdate\0slotLogout\0"
    "DAccount::Type\0slotLastSyncTimeUpdate\0"
    "datetime\0slotAccountStateChange\0"
    "slotFirstDayofWeekCurrentChanged\0index\0"
    "slotTimeTypeCurrentChanged\0"
    "slotAccountCurrentChanged\0"
    "slotTypeAddBtnClickded\0slotTypeImportBtnClickded\0"
    "slotSetUosSyncFreq\0freq\0slotUosManualSync\0"
    "slotNetworkStateChange\0"
    "DOANetWorkDBus::NetWorkState\0state\0"
    "slotSyncTagButtonUpdate\0"
    "slotSyncAccountStateUpdate"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CSettingDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      15,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   89,    2, 0x0a /* Public */,
       3,    0,   90,    2, 0x0a /* Public */,
       4,    1,   91,    2, 0x0a /* Public */,
       6,    1,   94,    2, 0x0a /* Public */,
       8,    0,   97,    2, 0x0a /* Public */,
       9,    1,   98,    2, 0x0a /* Public */,
      11,    1,  101,    2, 0x0a /* Public */,
      12,    1,  104,    2, 0x0a /* Public */,
      13,    0,  107,    2, 0x0a /* Public */,
      14,    0,  108,    2, 0x0a /* Public */,
      15,    1,  109,    2, 0x0a /* Public */,
      17,    0,  112,    2, 0x0a /* Public */,
      18,    1,  113,    2, 0x0a /* Public */,
      21,    0,  116,    2, 0x0a /* Public */,
      22,    1,  117,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 5,    2,
    QMetaType::Void, QMetaType::QString,    7,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   10,
    QMetaType::Void, QMetaType::Int,   10,
    QMetaType::Void, QMetaType::Int,   10,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   16,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 19,   20,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,

       0        // eod
};

void CSettingDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CSettingDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->slotGeneralSettingsUpdate(); break;
        case 1: _t->slotAccountUpdate(); break;
        case 2: _t->slotLogout((*reinterpret_cast< DAccount::Type(*)>(_a[1]))); break;
        case 3: _t->slotLastSyncTimeUpdate((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 4: _t->slotAccountStateChange(); break;
        case 5: _t->slotFirstDayofWeekCurrentChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->slotTimeTypeCurrentChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->slotAccountCurrentChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: _t->slotTypeAddBtnClickded(); break;
        case 9: _t->slotTypeImportBtnClickded(); break;
        case 10: _t->slotSetUosSyncFreq((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 11: _t->slotUosManualSync(); break;
        case 12: _t->slotNetworkStateChange((*reinterpret_cast< DOANetWorkDBus::NetWorkState(*)>(_a[1]))); break;
        case 13: _t->slotSyncTagButtonUpdate(); break;
        case 14: _t->slotSyncAccountStateUpdate((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject CSettingDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<DSettingsDialog::staticMetaObject>(),
    qt_meta_stringdata_CSettingDialog.data,
    qt_meta_data_CSettingDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CSettingDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CSettingDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CSettingDialog.stringdata0))
        return static_cast<void*>(this);
    return DSettingsDialog::qt_metacast(_clname);
}

int CSettingDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DSettingsDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 15;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
