// SPDX-FileCopyrightText: 2019 - 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: LGPL-3.0-or-later

#ifndef CREATEJSONDATA_H
#define CREATEJSONDATA_H

#include "jsondata.h"

class CreateJsonData : public JsonData
{
public:
    CreateJsonData();

private:
};

#endif // CREATEJSONDATA_H
