/****************************************************************************
** Meta object code from reading C++ file 'dhuangliservice.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/dbusservice/dhuangliservice.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dhuangliservice.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_DHuangliService_t {
    QByteArrayData data[15];
    char stringdata0[204];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DHuangliService_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DHuangliService_t qt_meta_stringdata_DHuangliService = {
    {
QT_MOC_LITERAL(0, 0, 15), // "DHuangliService"
QT_MOC_LITERAL(1, 16, 15), // "D-Bus Interface"
QT_MOC_LITERAL(2, 32, 29), // "com.deepin.dataserver.HuangLi"
QT_MOC_LITERAL(3, 62, 16), // "getFestivalMonth"
QT_MOC_LITERAL(4, 79, 0), // ""
QT_MOC_LITERAL(5, 80, 4), // "year"
QT_MOC_LITERAL(6, 85, 5), // "month"
QT_MOC_LITERAL(7, 91, 13), // "getHuangLiDay"
QT_MOC_LITERAL(8, 105, 3), // "day"
QT_MOC_LITERAL(9, 109, 15), // "getHuangLiMonth"
QT_MOC_LITERAL(10, 125, 4), // "fill"
QT_MOC_LITERAL(11, 130, 19), // "getLunarInfoBySolar"
QT_MOC_LITERAL(12, 150, 14), // "CaLunarDayInfo"
QT_MOC_LITERAL(13, 165, 21), // "getLunarMonthCalendar"
QT_MOC_LITERAL(14, 187, 16) // "CaLunarMonthInfo"

    },
    "DHuangliService\0D-Bus Interface\0"
    "com.deepin.dataserver.HuangLi\0"
    "getFestivalMonth\0\0year\0month\0getHuangLiDay\0"
    "day\0getHuangLiMonth\0fill\0getLunarInfoBySolar\0"
    "CaLunarDayInfo\0getLunarMonthCalendar\0"
    "CaLunarMonthInfo"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DHuangliService[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       5,   16, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,

 // slots: name, argc, parameters, tag, flags
       3,    2,   41,    4, 0x4a /* Public | isScriptable */,
       7,    3,   46,    4, 0x4a /* Public | isScriptable */,
       9,    3,   53,    4, 0x4a /* Public | isScriptable */,
      11,    3,   60,    4, 0x4a /* Public | isScriptable */,
      13,    3,   67,    4, 0x4a /* Public | isScriptable */,

 // slots: parameters
    QMetaType::QString, QMetaType::UInt, QMetaType::UInt,    5,    6,
    QMetaType::QString, QMetaType::UInt, QMetaType::UInt, QMetaType::UInt,    5,    6,    8,
    QMetaType::QString, QMetaType::UInt, QMetaType::UInt, QMetaType::Bool,    5,    6,   10,
    0x80000000 | 12, QMetaType::UInt, QMetaType::UInt, QMetaType::UInt,    5,    6,    8,
    0x80000000 | 14, QMetaType::UInt, QMetaType::UInt, QMetaType::Bool,    5,    6,   10,

       0        // eod
};

void DHuangliService::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DHuangliService *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { QString _r = _t->getFestivalMonth((*reinterpret_cast< quint32(*)>(_a[1])),(*reinterpret_cast< quint32(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 1: { QString _r = _t->getHuangLiDay((*reinterpret_cast< quint32(*)>(_a[1])),(*reinterpret_cast< quint32(*)>(_a[2])),(*reinterpret_cast< quint32(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 2: { QString _r = _t->getHuangLiMonth((*reinterpret_cast< quint32(*)>(_a[1])),(*reinterpret_cast< quint32(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 3: { CaLunarDayInfo _r = _t->getLunarInfoBySolar((*reinterpret_cast< quint32(*)>(_a[1])),(*reinterpret_cast< quint32(*)>(_a[2])),(*reinterpret_cast< quint32(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< CaLunarDayInfo*>(_a[0]) = std::move(_r); }  break;
        case 4: { CaLunarMonthInfo _r = _t->getLunarMonthCalendar((*reinterpret_cast< quint32(*)>(_a[1])),(*reinterpret_cast< quint32(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< CaLunarMonthInfo*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DHuangliService::staticMetaObject = { {
    QMetaObject::SuperData::link<DServiceBase::staticMetaObject>(),
    qt_meta_stringdata_DHuangliService.data,
    qt_meta_data_DHuangliService,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DHuangliService::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DHuangliService::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DHuangliService.stringdata0))
        return static_cast<void*>(this);
    return DServiceBase::qt_metacast(_clname);
}

int DHuangliService::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DServiceBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
