/****************************************************************************
** Meta object code from reading C++ file 'cgraphicsscene.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/view/cgraphicsscene.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'cgraphicsscene.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_CGraphicsScene_t {
    QByteArrayData data[15];
    char stringdata0[207];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CGraphicsScene_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CGraphicsScene_t qt_meta_stringdata_CGraphicsScene = {
    {
QT_MOC_LITERAL(0, 0, 14), // "CGraphicsScene"
QT_MOC_LITERAL(1, 15, 19), // "signalSwitchPrePage"
QT_MOC_LITERAL(2, 35, 0), // ""
QT_MOC_LITERAL(3, 36, 9), // "focusDate"
QT_MOC_LITERAL(4, 46, 12), // "isSwitchView"
QT_MOC_LITERAL(5, 59, 20), // "signalSwitchNextPage"
QT_MOC_LITERAL(6, 80, 16), // "signalSwitchView"
QT_MOC_LITERAL(7, 97, 12), // "setItemFocus"
QT_MOC_LITERAL(8, 110, 19), // "signalViewFocusInit"
QT_MOC_LITERAL(9, 130, 17), // "signalGotoDayView"
QT_MOC_LITERAL(10, 148, 4), // "date"
QT_MOC_LITERAL(11, 153, 17), // "signalContextMenu"
QT_MOC_LITERAL(12, 171, 11), // "CFocusItem*"
QT_MOC_LITERAL(13, 183, 4), // "item"
QT_MOC_LITERAL(14, 188, 18) // "signalsetNextFocus"

    },
    "CGraphicsScene\0signalSwitchPrePage\0\0"
    "focusDate\0isSwitchView\0signalSwitchNextPage\0"
    "signalSwitchView\0setItemFocus\0"
    "signalViewFocusInit\0signalGotoDayView\0"
    "date\0signalContextMenu\0CFocusItem*\0"
    "item\0signalsetNextFocus"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CGraphicsScene[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      10,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   64,    2, 0x06 /* Public */,
       1,    1,   69,    2, 0x26 /* Public | MethodCloned */,
       5,    2,   72,    2, 0x06 /* Public */,
       5,    1,   77,    2, 0x26 /* Public | MethodCloned */,
       6,    2,   80,    2, 0x06 /* Public */,
       6,    1,   85,    2, 0x26 /* Public | MethodCloned */,
       8,    0,   88,    2, 0x06 /* Public */,
       9,    1,   89,    2, 0x06 /* Public */,
      11,    1,   92,    2, 0x06 /* Public */,
      14,    0,   95,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QDate, QMetaType::Bool,    3,    4,
    QMetaType::Void, QMetaType::QDate,    3,
    QMetaType::Void, QMetaType::QDate, QMetaType::Bool,    3,    4,
    QMetaType::Void, QMetaType::QDate,    3,
    QMetaType::Void, QMetaType::QDate, QMetaType::Bool,    3,    7,
    QMetaType::Void, QMetaType::QDate,    3,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QDate,   10,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void,

       0        // eod
};

void CGraphicsScene::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CGraphicsScene *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->signalSwitchPrePage((*reinterpret_cast< const QDate(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 1: _t->signalSwitchPrePage((*reinterpret_cast< const QDate(*)>(_a[1]))); break;
        case 2: _t->signalSwitchNextPage((*reinterpret_cast< const QDate(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 3: _t->signalSwitchNextPage((*reinterpret_cast< const QDate(*)>(_a[1]))); break;
        case 4: _t->signalSwitchView((*reinterpret_cast< const QDate(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 5: _t->signalSwitchView((*reinterpret_cast< const QDate(*)>(_a[1]))); break;
        case 6: _t->signalViewFocusInit(); break;
        case 7: _t->signalGotoDayView((*reinterpret_cast< const QDate(*)>(_a[1]))); break;
        case 8: _t->signalContextMenu((*reinterpret_cast< CFocusItem*(*)>(_a[1]))); break;
        case 9: _t->signalsetNextFocus(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CGraphicsScene::*)(const QDate & , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CGraphicsScene::signalSwitchPrePage)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (CGraphicsScene::*)(const QDate & , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CGraphicsScene::signalSwitchNextPage)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (CGraphicsScene::*)(const QDate & , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CGraphicsScene::signalSwitchView)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (CGraphicsScene::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CGraphicsScene::signalViewFocusInit)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (CGraphicsScene::*)(const QDate & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CGraphicsScene::signalGotoDayView)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (CGraphicsScene::*)(CFocusItem * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CGraphicsScene::signalContextMenu)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (CGraphicsScene::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CGraphicsScene::signalsetNextFocus)) {
                *result = 9;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject CGraphicsScene::staticMetaObject = { {
    QMetaObject::SuperData::link<QGraphicsScene::staticMetaObject>(),
    qt_meta_stringdata_CGraphicsScene.data,
    qt_meta_data_CGraphicsScene,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CGraphicsScene::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CGraphicsScene::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CGraphicsScene.stringdata0))
        return static_cast<void*>(this);
    return QGraphicsScene::qt_metacast(_clname);
}

int CGraphicsScene::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QGraphicsScene::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void CGraphicsScene::signalSwitchPrePage(const QDate & _t1, bool _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 2
void CGraphicsScene::signalSwitchNextPage(const QDate & _t1, bool _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 4
void CGraphicsScene::signalSwitchView(const QDate & _t1, bool _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 6
void CGraphicsScene::signalViewFocusInit()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void CGraphicsScene::signalGotoDayView(const QDate & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void CGraphicsScene::signalContextMenu(CFocusItem * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void CGraphicsScene::signalsetNextFocus()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
