/****************************************************************************
** Meta object code from reading C++ file 'graphicsview.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/view/graphicsview.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QVector>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'graphicsview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_CGraphicsView_t {
    QByteArrayData data[14];
    char stringdata0[181];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CGraphicsView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CGraphicsView_t qt_meta_stringdata_CGraphicsView = {
    {
QT_MOC_LITERAL(0, 0, 13), // "CGraphicsView"
QT_MOC_LITERAL(1, 14, 15), // "signalsPosHours"
QT_MOC_LITERAL(2, 30, 0), // ""
QT_MOC_LITERAL(3, 31, 12), // "QVector<int>"
QT_MOC_LITERAL(4, 44, 4), // "vPos"
QT_MOC_LITERAL(5, 49, 6), // "vHours"
QT_MOC_LITERAL(6, 56, 15), // "currentTimeType"
QT_MOC_LITERAL(7, 72, 26), // "signalsCurrentScheduleDate"
QT_MOC_LITERAL(8, 99, 4), // "date"
QT_MOC_LITERAL(9, 104, 25), // "scrollBarValueChangedSlot"
QT_MOC_LITERAL(10, 130, 15), // "slotDoubleEvent"
QT_MOC_LITERAL(11, 146, 4), // "type"
QT_MOC_LITERAL(12, 151, 13), // "slotScrollBar"
QT_MOC_LITERAL(13, 165, 15) // "slotUpdateScene"

    },
    "CGraphicsView\0signalsPosHours\0\0"
    "QVector<int>\0vPos\0vHours\0currentTimeType\0"
    "signalsCurrentScheduleDate\0date\0"
    "scrollBarValueChangedSlot\0slotDoubleEvent\0"
    "type\0slotScrollBar\0slotUpdateScene"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CGraphicsView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    3,   49,    2, 0x06 /* Public */,
       1,    2,   56,    2, 0x26 /* Public | MethodCloned */,
       7,    1,   61,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       9,    0,   64,    2, 0x0a /* Public */,
      10,    1,   65,    2, 0x0a /* Public */,
      12,    0,   68,    2, 0x0a /* Public */,
      13,    0,   69,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 3, QMetaType::Int,    4,    5,    6,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 3,    4,    5,
    QMetaType::Void, QMetaType::QDate,    8,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   11,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void CGraphicsView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CGraphicsView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->signalsPosHours((*reinterpret_cast< QVector<int>(*)>(_a[1])),(*reinterpret_cast< QVector<int>(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 1: _t->signalsPosHours((*reinterpret_cast< QVector<int>(*)>(_a[1])),(*reinterpret_cast< QVector<int>(*)>(_a[2]))); break;
        case 2: _t->signalsCurrentScheduleDate((*reinterpret_cast< QDate(*)>(_a[1]))); break;
        case 3: _t->scrollBarValueChangedSlot(); break;
        case 4: _t->slotDoubleEvent((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->slotScrollBar(); break;
        case 6: _t->slotUpdateScene(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QVector<int> >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QVector<int> >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CGraphicsView::*)(QVector<int> , QVector<int> , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CGraphicsView::signalsPosHours)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (CGraphicsView::*)(QDate );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CGraphicsView::signalsCurrentScheduleDate)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject CGraphicsView::staticMetaObject = { {
    QMetaObject::SuperData::link<CWeekDayGraphicsview::staticMetaObject>(),
    qt_meta_stringdata_CGraphicsView.data,
    qt_meta_data_CGraphicsView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CGraphicsView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CGraphicsView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CGraphicsView.stringdata0))
        return static_cast<void*>(this);
    return CWeekDayGraphicsview::qt_metacast(_clname);
}

int CGraphicsView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = CWeekDayGraphicsview::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void CGraphicsView::signalsPosHours(QVector<int> _t1, QVector<int> _t2, int _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 2
void CGraphicsView::signalsCurrentScheduleDate(QDate _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
