/****************************************************************************
** Meta object code from reading C++ file 'accountitem.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/calendarData/accountitem.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'accountitem.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_AccountItem_t {
    QByteArrayData data[19];
    char stringdata0[391];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_AccountItem_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_AccountItem_t qt_meta_stringdata_AccountItem = {
    {
QT_MOC_LITERAL(0, 0, 11), // "AccountItem"
QT_MOC_LITERAL(1, 12, 23), // "signalAccountDataUpdate"
QT_MOC_LITERAL(2, 36, 0), // ""
QT_MOC_LITERAL(3, 37, 20), // "signalScheduleUpdate"
QT_MOC_LITERAL(4, 58, 24), // "signalScheduleTypeUpdate"
QT_MOC_LITERAL(5, 83, 26), // "signalSearchScheduleUpdate"
QT_MOC_LITERAL(6, 110, 18), // "signalCreateFinish"
QT_MOC_LITERAL(7, 129, 10), // "scheduleID"
QT_MOC_LITERAL(8, 140, 23), // "signalGetScheduleFinish"
QT_MOC_LITERAL(9, 164, 14), // "DSchedule::Ptr"
QT_MOC_LITERAL(10, 179, 24), // "slotGetAccountInfoFinish"
QT_MOC_LITERAL(11, 204, 13), // "DAccount::Ptr"
QT_MOC_LITERAL(12, 218, 29), // "slotGetScheduleTypeListFinish"
QT_MOC_LITERAL(13, 248, 19), // "DScheduleType::List"
QT_MOC_LITERAL(14, 268, 25), // "slotGetScheduleListFinish"
QT_MOC_LITERAL(15, 294, 27), // "QMap<QDate,DSchedule::List>"
QT_MOC_LITERAL(16, 322, 28), // "slotSearchScheduleListFinish"
QT_MOC_LITERAL(17, 351, 22), // "slotGetSysColorsFinish"
QT_MOC_LITERAL(18, 374, 16) // "DTypeColor::List"

    },
    "AccountItem\0signalAccountDataUpdate\0"
    "\0signalScheduleUpdate\0signalScheduleTypeUpdate\0"
    "signalSearchScheduleUpdate\0"
    "signalCreateFinish\0scheduleID\0"
    "signalGetScheduleFinish\0DSchedule::Ptr\0"
    "slotGetAccountInfoFinish\0DAccount::Ptr\0"
    "slotGetScheduleTypeListFinish\0"
    "DScheduleType::List\0slotGetScheduleListFinish\0"
    "QMap<QDate,DSchedule::List>\0"
    "slotSearchScheduleListFinish\0"
    "slotGetSysColorsFinish\0DTypeColor::List"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_AccountItem[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       6,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   69,    2, 0x06 /* Public */,
       3,    0,   70,    2, 0x06 /* Public */,
       4,    0,   71,    2, 0x06 /* Public */,
       5,    0,   72,    2, 0x06 /* Public */,
       6,    1,   73,    2, 0x06 /* Public */,
       8,    1,   76,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      10,    1,   79,    2, 0x0a /* Public */,
      12,    1,   82,    2, 0x0a /* Public */,
      14,    1,   85,    2, 0x0a /* Public */,
      16,    1,   88,    2, 0x0a /* Public */,
      17,    1,   91,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    7,
    QMetaType::Void, 0x80000000 | 9,    2,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void, 0x80000000 | 13,    2,
    QMetaType::Void, 0x80000000 | 15,    2,
    QMetaType::Void, 0x80000000 | 15,    2,
    QMetaType::Void, 0x80000000 | 18,    2,

       0        // eod
};

void AccountItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<AccountItem *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->signalAccountDataUpdate(); break;
        case 1: _t->signalScheduleUpdate(); break;
        case 2: _t->signalScheduleTypeUpdate(); break;
        case 3: _t->signalSearchScheduleUpdate(); break;
        case 4: _t->signalCreateFinish((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->signalGetScheduleFinish((*reinterpret_cast< const DSchedule::Ptr(*)>(_a[1]))); break;
        case 6: _t->slotGetAccountInfoFinish((*reinterpret_cast< DAccount::Ptr(*)>(_a[1]))); break;
        case 7: _t->slotGetScheduleTypeListFinish((*reinterpret_cast< DScheduleType::List(*)>(_a[1]))); break;
        case 8: _t->slotGetScheduleListFinish((*reinterpret_cast< QMap<QDate,DSchedule::List>(*)>(_a[1]))); break;
        case 9: _t->slotSearchScheduleListFinish((*reinterpret_cast< QMap<QDate,DSchedule::List>(*)>(_a[1]))); break;
        case 10: _t->slotGetSysColorsFinish((*reinterpret_cast< DTypeColor::List(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (AccountItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AccountItem::signalAccountDataUpdate)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (AccountItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AccountItem::signalScheduleUpdate)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (AccountItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AccountItem::signalScheduleTypeUpdate)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (AccountItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AccountItem::signalSearchScheduleUpdate)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (AccountItem::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AccountItem::signalCreateFinish)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (AccountItem::*)(const DSchedule::Ptr & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AccountItem::signalGetScheduleFinish)) {
                *result = 5;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject AccountItem::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_AccountItem.data,
    qt_meta_data_AccountItem,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *AccountItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AccountItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_AccountItem.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int AccountItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void AccountItem::signalAccountDataUpdate()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void AccountItem::signalScheduleUpdate()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void AccountItem::signalScheduleTypeUpdate()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void AccountItem::signalSearchScheduleUpdate()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void AccountItem::signalCreateFinish(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void AccountItem::signalGetScheduleFinish(const DSchedule::Ptr & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
