/****************************************************************************
** Meta object code from reading C++ file 'daccountservice.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/dbusservice/daccountservice.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'daccountservice.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_DAccountService_t {
    QByteArrayData data[33];
    char stringdata0[518];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DAccountService_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DAccountService_t qt_meta_stringdata_DAccountService = {
    {
QT_MOC_LITERAL(0, 0, 15), // "DAccountService"
QT_MOC_LITERAL(1, 16, 15), // "D-Bus Interface"
QT_MOC_LITERAL(2, 32, 38), // "com.deepin.dataserver.Calenda..."
QT_MOC_LITERAL(3, 71, 14), // "scheduleUpdate"
QT_MOC_LITERAL(4, 86, 0), // ""
QT_MOC_LITERAL(5, 87, 18), // "scheduleTypeUpdate"
QT_MOC_LITERAL(6, 106, 14), // "getAccountInfo"
QT_MOC_LITERAL(7, 121, 19), // "getScheduleTypeList"
QT_MOC_LITERAL(8, 141, 19), // "getScheduleTypeByID"
QT_MOC_LITERAL(9, 161, 6), // "typeID"
QT_MOC_LITERAL(10, 168, 18), // "createScheduleType"
QT_MOC_LITERAL(11, 187, 8), // "typeInfo"
QT_MOC_LITERAL(12, 196, 18), // "updateScheduleType"
QT_MOC_LITERAL(13, 215, 22), // "deleteScheduleTypeByID"
QT_MOC_LITERAL(14, 238, 18), // "scheduleTypeByUsed"
QT_MOC_LITERAL(15, 257, 14), // "createSchedule"
QT_MOC_LITERAL(16, 272, 12), // "scheduleInfo"
QT_MOC_LITERAL(17, 285, 14), // "updateSchedule"
QT_MOC_LITERAL(18, 300, 23), // "getScheduleByScheduleID"
QT_MOC_LITERAL(19, 324, 10), // "scheduleID"
QT_MOC_LITERAL(20, 335, 26), // "deleteScheduleByScheduleID"
QT_MOC_LITERAL(21, 362, 27), // "querySchedulesWithParameter"
QT_MOC_LITERAL(22, 390, 6), // "params"
QT_MOC_LITERAL(23, 397, 14), // "exportSchedule"
QT_MOC_LITERAL(24, 412, 11), // "icsFilePath"
QT_MOC_LITERAL(25, 424, 14), // "importSchedule"
QT_MOC_LITERAL(26, 439, 11), // "cleanExists"
QT_MOC_LITERAL(27, 451, 12), // "getSysColors"
QT_MOC_LITERAL(28, 464, 8), // "isExpand"
QT_MOC_LITERAL(29, 473, 12), // "accountState"
QT_MOC_LITERAL(30, 486, 9), // "syncState"
QT_MOC_LITERAL(31, 496, 8), // "syncFreq"
QT_MOC_LITERAL(32, 505, 12) // "dtLastUpdate"

    },
    "DAccountService\0D-Bus Interface\0"
    "com.deepin.dataserver.Calendar.Account\0"
    "scheduleUpdate\0\0scheduleTypeUpdate\0"
    "getAccountInfo\0getScheduleTypeList\0"
    "getScheduleTypeByID\0typeID\0"
    "createScheduleType\0typeInfo\0"
    "updateScheduleType\0deleteScheduleTypeByID\0"
    "scheduleTypeByUsed\0createSchedule\0"
    "scheduleInfo\0updateSchedule\0"
    "getScheduleByScheduleID\0scheduleID\0"
    "deleteScheduleByScheduleID\0"
    "querySchedulesWithParameter\0params\0"
    "exportSchedule\0icsFilePath\0importSchedule\0"
    "cleanExists\0getSysColors\0isExpand\0"
    "accountState\0syncState\0syncFreq\0"
    "dtLastUpdate"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DAccountService[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      17,   16, // methods
       5,  148, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,  101,    4, 0x06 /* Public */,
       5,    0,  102,    4, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    0,  103,    4, 0x4a /* Public | isScriptable */,
       7,    0,  104,    4, 0x4a /* Public | isScriptable */,
       8,    1,  105,    4, 0x4a /* Public | isScriptable */,
      10,    1,  108,    4, 0x4a /* Public | isScriptable */,
      12,    1,  111,    4, 0x4a /* Public | isScriptable */,
      13,    1,  114,    4, 0x4a /* Public | isScriptable */,
      14,    1,  117,    4, 0x4a /* Public | isScriptable */,
      15,    1,  120,    4, 0x4a /* Public | isScriptable */,
      17,    1,  123,    4, 0x4a /* Public | isScriptable */,
      18,    1,  126,    4, 0x4a /* Public | isScriptable */,
      20,    1,  129,    4, 0x4a /* Public | isScriptable */,
      21,    1,  132,    4, 0x4a /* Public | isScriptable */,
      23,    2,  135,    4, 0x4a /* Public | isScriptable */,
      25,    3,  140,    4, 0x4a /* Public | isScriptable */,
      27,    0,  147,    4, 0x4a /* Public | isScriptable */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::QString,
    QMetaType::QString,
    QMetaType::QString, QMetaType::QString,    9,
    QMetaType::QString, QMetaType::QString,   11,
    QMetaType::Bool, QMetaType::QString,   11,
    QMetaType::Bool, QMetaType::QString,    9,
    QMetaType::Bool, QMetaType::QString,    9,
    QMetaType::QString, QMetaType::QString,   16,
    QMetaType::Bool, QMetaType::QString,   16,
    QMetaType::QString, QMetaType::QString,   19,
    QMetaType::Bool, QMetaType::QString,   19,
    QMetaType::QString, QMetaType::QString,   22,
    QMetaType::Bool, QMetaType::QString, QMetaType::QString,   24,    9,
    QMetaType::Bool, QMetaType::QString, QMetaType::QString, QMetaType::Bool,   24,    9,   26,
    QMetaType::QString,

 // properties: name, type, flags
      28, QMetaType::Bool, 0x00095003,
      29, QMetaType::Int, 0x00095103,
      30, QMetaType::Int, 0x00095001,
      31, QMetaType::QString, 0x00095103,
      32, QMetaType::QString, 0x00095001,

       0        // eod
};

void DAccountService::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DAccountService *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->scheduleUpdate(); break;
        case 1: _t->scheduleTypeUpdate(); break;
        case 2: { QString _r = _t->getAccountInfo();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 3: { QString _r = _t->getScheduleTypeList();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 4: { QString _r = _t->getScheduleTypeByID((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 5: { QString _r = _t->createScheduleType((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 6: { bool _r = _t->updateScheduleType((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 7: { bool _r = _t->deleteScheduleTypeByID((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 8: { bool _r = _t->scheduleTypeByUsed((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 9: { QString _r = _t->createSchedule((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 10: { bool _r = _t->updateSchedule((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 11: { QString _r = _t->getScheduleByScheduleID((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 12: { bool _r = _t->deleteScheduleByScheduleID((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 13: { QString _r = _t->querySchedulesWithParameter((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 14: { bool _r = _t->exportSchedule((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 15: { bool _r = _t->importSchedule((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const bool(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 16: { QString _r = _t->getSysColors();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DAccountService::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DAccountService::scheduleUpdate)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (DAccountService::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DAccountService::scheduleTypeUpdate)) {
                *result = 1;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<DAccountService *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->getExpand(); break;
        case 1: *reinterpret_cast< int*>(_v) = _t->getAccountState(); break;
        case 2: *reinterpret_cast< int*>(_v) = _t->getSyncState(); break;
        case 3: *reinterpret_cast< QString*>(_v) = _t->getSyncFreq(); break;
        case 4: *reinterpret_cast< QString*>(_v) = _t->getDtLastUpdate(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<DAccountService *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setExpand(*reinterpret_cast< bool*>(_v)); break;
        case 1: _t->setAccountState(*reinterpret_cast< int*>(_v)); break;
        case 3: _t->setSyncFreq(*reinterpret_cast< QString*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject DAccountService::staticMetaObject = { {
    QMetaObject::SuperData::link<DServiceBase::staticMetaObject>(),
    qt_meta_stringdata_DAccountService.data,
    qt_meta_data_DAccountService,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DAccountService::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DAccountService::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DAccountService.stringdata0))
        return static_cast<void*>(this);
    return DServiceBase::qt_metacast(_clname);
}

int DAccountService::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DServiceBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 17;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 5;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void DAccountService::scheduleUpdate()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void DAccountService::scheduleTypeUpdate()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
