/****************************************************************************
** Meta object code from reading C++ file 'scheduledlg.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/dialog/scheduledlg.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'scheduledlg.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_CScheduleDlg_t {
    QByteArrayData data[31];
    char stringdata0[489];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CScheduleDlg_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CScheduleDlg_t qt_meta_stringdata_CScheduleDlg = {
    {
QT_MOC_LITERAL(0, 0, 12), // "CScheduleDlg"
QT_MOC_LITERAL(1, 13, 20), // "signalScheduleUpdate"
QT_MOC_LITERAL(2, 34, 0), // ""
QT_MOC_LITERAL(3, 35, 2), // "id"
QT_MOC_LITERAL(4, 38, 19), // "slotBeginTimeChange"
QT_MOC_LITERAL(5, 58, 17), // "slotEndTimeChange"
QT_MOC_LITERAL(6, 76, 17), // "slotEndDateChange"
QT_MOC_LITERAL(7, 94, 4), // "date"
QT_MOC_LITERAL(8, 99, 19), // "slotRadioBtnClicked"
QT_MOC_LITERAL(9, 119, 5), // "btnId"
QT_MOC_LITERAL(10, 125, 21), // "slotBtnAddItemClicked"
QT_MOC_LITERAL(11, 147, 23), // "slotTypeEditTextChanged"
QT_MOC_LITERAL(12, 171, 22), // "slotTypeRpeatactivated"
QT_MOC_LITERAL(13, 194, 5), // "index"
QT_MOC_LITERAL(14, 200, 11), // "slotBtClick"
QT_MOC_LITERAL(15, 212, 11), // "buttonIndex"
QT_MOC_LITERAL(16, 224, 10), // "buttonName"
QT_MOC_LITERAL(17, 235, 14), // "slotTextChange"
QT_MOC_LITERAL(18, 250, 23), // "slotendrepeatTextchange"
QT_MOC_LITERAL(19, 274, 17), // "slotBDateEidtInfo"
QT_MOC_LITERAL(20, 292, 22), // "slotallDayStateChanged"
QT_MOC_LITERAL(21, 315, 5), // "state"
QT_MOC_LITERAL(22, 321, 19), // "slotbRpeatactivated"
QT_MOC_LITERAL(23, 341, 19), // "sloteRpeatactivated"
QT_MOC_LITERAL(24, 361, 30), // "slotJobComboBoxEditingFinished"
QT_MOC_LITERAL(25, 392, 22), // "slotAccoutBoxActivated"
QT_MOC_LITERAL(26, 415, 4), // "text"
QT_MOC_LITERAL(27, 420, 12), // "signalLogout"
QT_MOC_LITERAL(28, 433, 14), // "DAccount::Type"
QT_MOC_LITERAL(29, 448, 17), // "slotAccountUpdate"
QT_MOC_LITERAL(30, 466, 22) // "slotAccountStateChange"

    },
    "CScheduleDlg\0signalScheduleUpdate\0\0"
    "id\0slotBeginTimeChange\0slotEndTimeChange\0"
    "slotEndDateChange\0date\0slotRadioBtnClicked\0"
    "btnId\0slotBtnAddItemClicked\0"
    "slotTypeEditTextChanged\0slotTypeRpeatactivated\0"
    "index\0slotBtClick\0buttonIndex\0buttonName\0"
    "slotTextChange\0slotendrepeatTextchange\0"
    "slotBDateEidtInfo\0slotallDayStateChanged\0"
    "state\0slotbRpeatactivated\0sloteRpeatactivated\0"
    "slotJobComboBoxEditingFinished\0"
    "slotAccoutBoxActivated\0text\0signalLogout\0"
    "DAccount::Type\0slotAccountUpdate\0"
    "slotAccountStateChange"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CScheduleDlg[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      21,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  119,    2, 0x06 /* Public */,
       1,    0,  122,    2, 0x26 /* Public | MethodCloned */,

 // slots: name, argc, parameters, tag, flags
       4,    0,  123,    2, 0x0a /* Public */,
       5,    0,  124,    2, 0x0a /* Public */,
       6,    1,  125,    2, 0x0a /* Public */,
       8,    1,  128,    2, 0x0a /* Public */,
      10,    0,  131,    2, 0x0a /* Public */,
      11,    1,  132,    2, 0x0a /* Public */,
      12,    1,  135,    2, 0x0a /* Public */,
      14,    2,  138,    2, 0x0a /* Public */,
      17,    0,  143,    2, 0x0a /* Public */,
      18,    0,  144,    2, 0x0a /* Public */,
      19,    1,  145,    2, 0x0a /* Public */,
      20,    1,  148,    2, 0x0a /* Public */,
      22,    1,  151,    2, 0x0a /* Public */,
      23,    1,  154,    2, 0x0a /* Public */,
      24,    0,  157,    2, 0x0a /* Public */,
      25,    1,  158,    2, 0x0a /* Public */,
      27,    1,  161,    2, 0x0a /* Public */,
      29,    0,  164,    2, 0x0a /* Public */,
      30,    0,  165,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QDate,    7,
    QMetaType::Void, QMetaType::Int,    9,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Int,   13,
    QMetaType::Void, QMetaType::Int, QMetaType::QString,   15,   16,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QDate,    7,
    QMetaType::Void, QMetaType::Int,   21,
    QMetaType::Void, QMetaType::Int,   13,
    QMetaType::Void, QMetaType::Int,   13,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   26,
    QMetaType::Void, 0x80000000 | 28,    2,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void CScheduleDlg::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CScheduleDlg *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->signalScheduleUpdate((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->signalScheduleUpdate(); break;
        case 2: _t->slotBeginTimeChange(); break;
        case 3: _t->slotEndTimeChange(); break;
        case 4: _t->slotEndDateChange((*reinterpret_cast< const QDate(*)>(_a[1]))); break;
        case 5: _t->slotRadioBtnClicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->slotBtnAddItemClicked(); break;
        case 7: _t->slotTypeEditTextChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 8: _t->slotTypeRpeatactivated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 9: _t->slotBtClick((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 10: _t->slotTextChange(); break;
        case 11: _t->slotendrepeatTextchange(); break;
        case 12: _t->slotBDateEidtInfo((*reinterpret_cast< const QDate(*)>(_a[1]))); break;
        case 13: _t->slotallDayStateChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 14: _t->slotbRpeatactivated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 15: _t->sloteRpeatactivated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 16: _t->slotJobComboBoxEditingFinished(); break;
        case 17: _t->slotAccoutBoxActivated((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 18: _t->signalLogout((*reinterpret_cast< DAccount::Type(*)>(_a[1]))); break;
        case 19: _t->slotAccountUpdate(); break;
        case 20: _t->slotAccountStateChange(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CScheduleDlg::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CScheduleDlg::signalScheduleUpdate)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject CScheduleDlg::staticMetaObject = { {
    QMetaObject::SuperData::link<DCalendarDDialog::staticMetaObject>(),
    qt_meta_stringdata_CScheduleDlg.data,
    qt_meta_data_CScheduleDlg,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CScheduleDlg::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CScheduleDlg::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CScheduleDlg.stringdata0))
        return static_cast<void*>(this);
    return DCalendarDDialog::qt_metacast(_clname);
}

int CScheduleDlg::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DCalendarDDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 21;
    }
    return _id;
}

// SIGNAL 0
void CScheduleDlg::signalScheduleUpdate(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
