/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp -m -c ControlCenterProxy -p controlCenterProxy /build/deepin-calendar/src/dde-calendar/calendar-client/src/dbus/org.deepin.dde.ControlCenter.xml
 *
 * qdbusxml2cpp is Copyright (C) 2023 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef CONTROLCENTERPROXY_H
#define CONTROLCENTERPROXY_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.deepin.dde.ControlCenter1
 */
class ControlCenterProxy: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.deepin.dde.ControlCenter1"; }

public:
    ControlCenterProxy(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~ControlCenterProxy();

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<> ShowPage(const QString &url)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(url);
        return asyncCallWithArgumentList(QStringLiteral("ShowPage"), argumentList);
    }

Q_SIGNALS: // SIGNALS
};

namespace org {
  namespace deepin {
    namespace dde {
      typedef ::ControlCenterProxy ControlCenter1;
    }
  }
}
#endif
