/****************************************************************************
** Meta object code from reading C++ file 'dunioniddbus.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/dbus/dunioniddbus.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dunioniddbus.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_DUnionIDDbus_t {
    QByteArrayData data[17];
    char stringdata0[199];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DUnionIDDbus_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DUnionIDDbus_t qt_meta_stringdata_DUnionIDDbus = {
    {
QT_MOC_LITERAL(0, 0, 12), // "DUnionIDDbus"
QT_MOC_LITERAL(1, 13, 5), // "login"
QT_MOC_LITERAL(2, 19, 19), // "QDBusPendingReply<>"
QT_MOC_LITERAL(3, 39, 0), // ""
QT_MOC_LITERAL(4, 40, 6), // "logout"
QT_MOC_LITERAL(5, 47, 6), // "Upload"
QT_MOC_LITERAL(6, 54, 26), // "QDBusPendingReply<QString>"
QT_MOC_LITERAL(7, 81, 3), // "key"
QT_MOC_LITERAL(8, 85, 8), // "Download"
QT_MOC_LITERAL(9, 94, 5), // "path1"
QT_MOC_LITERAL(10, 100, 6), // "Delete"
QT_MOC_LITERAL(11, 107, 8), // "Metadata"
QT_MOC_LITERAL(12, 116, 29), // "QDBusPendingReply<QByteArray>"
QT_MOC_LITERAL(13, 146, 12), // "SwitcherDump"
QT_MOC_LITERAL(14, 159, 11), // "SwitcherGet"
QT_MOC_LITERAL(15, 171, 23), // "QDBusPendingReply<bool>"
QT_MOC_LITERAL(16, 195, 3) // "arg"

    },
    "DUnionIDDbus\0login\0QDBusPendingReply<>\0"
    "\0logout\0Upload\0QDBusPendingReply<QString>\0"
    "key\0Download\0path1\0Delete\0Metadata\0"
    "QDBusPendingReply<QByteArray>\0"
    "SwitcherDump\0SwitcherGet\0"
    "QDBusPendingReply<bool>\0arg"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DUnionIDDbus[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   54,    3, 0x0a /* Public */,
       4,    0,   55,    3, 0x0a /* Public */,
       5,    1,   56,    3, 0x0a /* Public */,
       8,    2,   59,    3, 0x0a /* Public */,
      10,    1,   64,    3, 0x0a /* Public */,
      11,    1,   67,    3, 0x0a /* Public */,
      13,    0,   70,    3, 0x0a /* Public */,
      14,    1,   71,    3, 0x0a /* Public */,

 // slots: parameters
    0x80000000 | 2,
    0x80000000 | 2,
    0x80000000 | 6, QMetaType::QString,    7,
    0x80000000 | 6, QMetaType::QString, QMetaType::QString,    7,    9,
    0x80000000 | 6, QMetaType::QString,    7,
    0x80000000 | 12, QMetaType::QString,    7,
    0x80000000 | 6,
    0x80000000 | 15, QMetaType::QString,   16,

       0        // eod
};

void DUnionIDDbus::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DUnionIDDbus *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { QDBusPendingReply<> _r = _t->login();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 1: { QDBusPendingReply<> _r = _t->logout();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 2: { QDBusPendingReply<QString> _r = _t->Upload((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 3: { QDBusPendingReply<QString> _r = _t->Download((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 4: { QDBusPendingReply<QString> _r = _t->Delete((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 5: { QDBusPendingReply<QByteArray> _r = _t->Metadata((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<QByteArray>*>(_a[0]) = std::move(_r); }  break;
        case 6: { QDBusPendingReply<QString> _r = _t->SwitcherDump();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 7: { QDBusPendingReply<bool> _r = _t->SwitcherGet((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DUnionIDDbus::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_meta_stringdata_DUnionIDDbus.data,
    qt_meta_data_DUnionIDDbus,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DUnionIDDbus::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DUnionIDDbus::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DUnionIDDbus.stringdata0))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int DUnionIDDbus::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 8;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
