// SPDX-FileCopyrightText: 2019 - 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: LGPL-3.0-or-later

#include "canceljsondata.h"
#include <QJsonObject>

#include "../globaldef.h"
cancelJsonData::cancelJsonData()
    : JsonData()
{
}

cancelJsonData::~cancelJsonData()
{
}
