/****************************************************************************
** Meta object code from reading C++ file 'daccountmodule.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/calendarDataManager/daccountmodule.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'daccountmodule.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_DAccountModule_t {
    QByteArrayData data[18];
    char stringdata0[290];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DAccountModule_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DAccountModule_t qt_meta_stringdata_DAccountModule = {
    {
QT_MOC_LITERAL(0, 0, 14), // "DAccountModule"
QT_MOC_LITERAL(1, 15, 20), // "signalScheduleUpdate"
QT_MOC_LITERAL(2, 36, 0), // ""
QT_MOC_LITERAL(3, 37, 24), // "signalScheduleTypeUpdate"
QT_MOC_LITERAL(4, 62, 23), // "signalCloseNotification"
QT_MOC_LITERAL(5, 86, 8), // "notifyID"
QT_MOC_LITERAL(6, 95, 18), // "signalAccountState"
QT_MOC_LITERAL(7, 114, 15), // "signalSyncState"
QT_MOC_LITERAL(8, 130, 18), // "signalDtLastUpdate"
QT_MOC_LITERAL(9, 149, 19), // "signalSettingChange"
QT_MOC_LITERAL(10, 169, 18), // "signalSyncFinished"
QT_MOC_LITERAL(11, 188, 16), // "slotOpenCalendar"
QT_MOC_LITERAL(12, 205, 7), // "alarmID"
QT_MOC_LITERAL(13, 213, 13), // "slotSyncState"
QT_MOC_LITERAL(14, 227, 9), // "syncState"
QT_MOC_LITERAL(15, 237, 14), // "slotDateUpdate"
QT_MOC_LITERAL(16, 252, 26), // "DDataSyncBase::UpdateTypes"
QT_MOC_LITERAL(17, 279, 10) // "updateType"

    },
    "DAccountModule\0signalScheduleUpdate\0"
    "\0signalScheduleTypeUpdate\0"
    "signalCloseNotification\0notifyID\0"
    "signalAccountState\0signalSyncState\0"
    "signalDtLastUpdate\0signalSettingChange\0"
    "signalSyncFinished\0slotOpenCalendar\0"
    "alarmID\0slotSyncState\0syncState\0"
    "slotDateUpdate\0DDataSyncBase::UpdateTypes\0"
    "updateType"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DAccountModule[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       8,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   69,    2, 0x06 /* Public */,
       3,    0,   70,    2, 0x06 /* Public */,
       4,    1,   71,    2, 0x06 /* Public */,
       6,    0,   74,    2, 0x06 /* Public */,
       7,    0,   75,    2, 0x06 /* Public */,
       8,    0,   76,    2, 0x06 /* Public */,
       9,    0,   77,    2, 0x06 /* Public */,
      10,    1,   78,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      11,    1,   81,    2, 0x0a /* Public */,
      13,    1,   84,    2, 0x0a /* Public */,
      15,    1,   87,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::ULongLong,    5,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,   12,
    QMetaType::Void, QMetaType::Int,   14,
    QMetaType::Void, 0x80000000 | 16,   17,

       0        // eod
};

void DAccountModule::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DAccountModule *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->signalScheduleUpdate(); break;
        case 1: _t->signalScheduleTypeUpdate(); break;
        case 2: _t->signalCloseNotification((*reinterpret_cast< quint64(*)>(_a[1]))); break;
        case 3: _t->signalAccountState(); break;
        case 4: _t->signalSyncState(); break;
        case 5: _t->signalDtLastUpdate(); break;
        case 6: _t->signalSettingChange(); break;
        case 7: _t->signalSyncFinished((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: _t->slotOpenCalendar((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 9: _t->slotSyncState((*reinterpret_cast< const int(*)>(_a[1]))); break;
        case 10: _t->slotDateUpdate((*reinterpret_cast< const DDataSyncBase::UpdateTypes(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DAccountModule::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DAccountModule::signalScheduleUpdate)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (DAccountModule::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DAccountModule::signalScheduleTypeUpdate)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (DAccountModule::*)(quint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DAccountModule::signalCloseNotification)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (DAccountModule::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DAccountModule::signalAccountState)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (DAccountModule::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DAccountModule::signalSyncState)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (DAccountModule::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DAccountModule::signalDtLastUpdate)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (DAccountModule::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DAccountModule::signalSettingChange)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (DAccountModule::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DAccountModule::signalSyncFinished)) {
                *result = 7;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DAccountModule::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_DAccountModule.data,
    qt_meta_data_DAccountModule,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DAccountModule::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DAccountModule::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DAccountModule.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int DAccountModule::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void DAccountModule::signalScheduleUpdate()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void DAccountModule::signalScheduleTypeUpdate()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void DAccountModule::signalCloseNotification(quint64 _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void DAccountModule::signalAccountState()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void DAccountModule::signalSyncState()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void DAccountModule::signalDtLastUpdate()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void DAccountModule::signalSettingChange()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void DAccountModule::signalSyncFinished(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
