/****************************************************************************
** Meta object code from reading C++ file 'incidence.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/incidence.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'incidence.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_KCalendarCore__Incidence_t {
    QByteArrayData data[31];
    char stringdata0[424];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KCalendarCore__Incidence_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KCalendarCore__Incidence_t qt_meta_stringdata_KCalendarCore__Incidence = {
    {
QT_MOC_LITERAL(0, 0, 24), // "KCalendarCore::Incidence"
QT_MOC_LITERAL(1, 25, 11), // "description"
QT_MOC_LITERAL(2, 37, 7), // "summary"
QT_MOC_LITERAL(3, 45, 8), // "location"
QT_MOC_LITERAL(4, 54, 6), // "hasGeo"
QT_MOC_LITERAL(5, 61, 11), // "geoLatitude"
QT_MOC_LITERAL(6, 73, 12), // "geoLongitude"
QT_MOC_LITERAL(7, 86, 10), // "categories"
QT_MOC_LITERAL(8, 97, 8), // "priority"
QT_MOC_LITERAL(9, 106, 7), // "created"
QT_MOC_LITERAL(10, 114, 7), // "secrecy"
QT_MOC_LITERAL(11, 122, 33), // "KCalendarCore::Incidence::Sec..."
QT_MOC_LITERAL(12, 156, 6), // "status"
QT_MOC_LITERAL(13, 163, 32), // "KCalendarCore::Incidence::Status"
QT_MOC_LITERAL(14, 196, 11), // "attachments"
QT_MOC_LITERAL(15, 208, 11), // "conferences"
QT_MOC_LITERAL(16, 220, 6), // "Status"
QT_MOC_LITERAL(17, 227, 10), // "StatusNone"
QT_MOC_LITERAL(18, 238, 15), // "StatusTentative"
QT_MOC_LITERAL(19, 254, 15), // "StatusConfirmed"
QT_MOC_LITERAL(20, 270, 15), // "StatusCompleted"
QT_MOC_LITERAL(21, 286, 17), // "StatusNeedsAction"
QT_MOC_LITERAL(22, 304, 14), // "StatusCanceled"
QT_MOC_LITERAL(23, 319, 15), // "StatusInProcess"
QT_MOC_LITERAL(24, 335, 11), // "StatusDraft"
QT_MOC_LITERAL(25, 347, 11), // "StatusFinal"
QT_MOC_LITERAL(26, 359, 7), // "StatusX"
QT_MOC_LITERAL(27, 367, 7), // "Secrecy"
QT_MOC_LITERAL(28, 375, 13), // "SecrecyPublic"
QT_MOC_LITERAL(29, 389, 14), // "SecrecyPrivate"
QT_MOC_LITERAL(30, 404, 19) // "SecrecyConfidential"

    },
    "KCalendarCore::Incidence\0description\0"
    "summary\0location\0hasGeo\0geoLatitude\0"
    "geoLongitude\0categories\0priority\0"
    "created\0secrecy\0KCalendarCore::Incidence::Secrecy\0"
    "status\0KCalendarCore::Incidence::Status\0"
    "attachments\0conferences\0Status\0"
    "StatusNone\0StatusTentative\0StatusConfirmed\0"
    "StatusCompleted\0StatusNeedsAction\0"
    "StatusCanceled\0StatusInProcess\0"
    "StatusDraft\0StatusFinal\0StatusX\0Secrecy\0"
    "SecrecyPublic\0SecrecyPrivate\0"
    "SecrecyConfidential"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KCalendarCore__Incidence[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
      13,   14, // properties
       2,   53, // enums/sets
       0,    0, // constructors
       4,       // flags
       0,       // signalCount

 // properties: name, type, flags
       1, QMetaType::QString, 0x00095103,
       2, QMetaType::QString, 0x00095103,
       3, QMetaType::QString, 0x00095103,
       4, QMetaType::Bool, 0x00095103,
       5, QMetaType::Float, 0x00095103,
       6, QMetaType::Float, 0x00095103,
       7, QMetaType::QStringList, 0x00095103,
       8, QMetaType::Int, 0x00095103,
       9, QMetaType::QDateTime, 0x00095103,
      10, 0x80000000 | 11, 0x0009510b,
      12, 0x80000000 | 13, 0x0009510b,
      14, QMetaType::QVariantList, 0x00095001,
      15, QMetaType::QVariantList, 0x00095001,

 // enums: name, alias, flags, count, data
      16,   16, 0x0,   10,   63,
      27,   27, 0x0,    3,   83,

 // enum data: key, value
      17, uint(KCalendarCore::Incidence::StatusNone),
      18, uint(KCalendarCore::Incidence::StatusTentative),
      19, uint(KCalendarCore::Incidence::StatusConfirmed),
      20, uint(KCalendarCore::Incidence::StatusCompleted),
      21, uint(KCalendarCore::Incidence::StatusNeedsAction),
      22, uint(KCalendarCore::Incidence::StatusCanceled),
      23, uint(KCalendarCore::Incidence::StatusInProcess),
      24, uint(KCalendarCore::Incidence::StatusDraft),
      25, uint(KCalendarCore::Incidence::StatusFinal),
      26, uint(KCalendarCore::Incidence::StatusX),
      28, uint(KCalendarCore::Incidence::SecrecyPublic),
      29, uint(KCalendarCore::Incidence::SecrecyPrivate),
      30, uint(KCalendarCore::Incidence::SecrecyConfidential),

       0        // eod
};

void KCalendarCore::Incidence::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{

#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty) {
        auto *_t = reinterpret_cast<Incidence *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QString*>(_v) = _t->description(); break;
        case 1: *reinterpret_cast< QString*>(_v) = _t->summary(); break;
        case 2: *reinterpret_cast< QString*>(_v) = _t->location(); break;
        case 3: *reinterpret_cast< bool*>(_v) = _t->hasGeo(); break;
        case 4: *reinterpret_cast< float*>(_v) = _t->geoLatitude(); break;
        case 5: *reinterpret_cast< float*>(_v) = _t->geoLongitude(); break;
        case 6: *reinterpret_cast< QStringList*>(_v) = _t->categories(); break;
        case 7: *reinterpret_cast< int*>(_v) = _t->priority(); break;
        case 8: *reinterpret_cast< QDateTime*>(_v) = _t->created(); break;
        case 9: *reinterpret_cast< KCalendarCore::Incidence::Secrecy*>(_v) = _t->secrecy(); break;
        case 10: *reinterpret_cast< KCalendarCore::Incidence::Status*>(_v) = _t->status(); break;
        case 11: *reinterpret_cast< QVariantList*>(_v) = _t->attachmentsVariant(); break;
        case 12: *reinterpret_cast< QVariantList*>(_v) = _t->conferencesVariant(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = reinterpret_cast<Incidence *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setDescription(*reinterpret_cast< QString*>(_v)); break;
        case 1: _t->setSummary(*reinterpret_cast< QString*>(_v)); break;
        case 2: _t->setLocation(*reinterpret_cast< QString*>(_v)); break;
        case 3: _t->setHasGeo(*reinterpret_cast< bool*>(_v)); break;
        case 4: _t->setGeoLatitude(*reinterpret_cast< float*>(_v)); break;
        case 5: _t->setGeoLongitude(*reinterpret_cast< float*>(_v)); break;
        case 6: _t->setCategories(*reinterpret_cast< QStringList*>(_v)); break;
        case 7: _t->setPriority(*reinterpret_cast< int*>(_v)); break;
        case 8: _t->setCreated(*reinterpret_cast< QDateTime*>(_v)); break;
        case 9: _t->setSecrecy(*reinterpret_cast< KCalendarCore::Incidence::Secrecy*>(_v)); break;
        case 10: _t->setStatus(*reinterpret_cast< KCalendarCore::Incidence::Status*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject KCalendarCore::Incidence::staticMetaObject = { {
    QtPrivate::MetaObjectForType<IncidenceBase>::value(),
    qt_meta_stringdata_KCalendarCore__Incidence.data,
    qt_meta_data_KCalendarCore__Incidence,
    qt_static_metacall,
    nullptr,
    nullptr
} };

QT_WARNING_POP
QT_END_MOC_NAMESPACE
