/****************************************************************************
** Meta object code from reading C++ file 'monthview.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/widget/monthWidget/monthview.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'monthview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_CMonthView_t {
    QByteArrayData data[16];
    char stringdata0[209];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CMonthView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CMonthView_t qt_meta_stringdata_CMonthView = {
    {
QT_MOC_LITERAL(0, 0, 10), // "CMonthView"
QT_MOC_LITERAL(1, 11, 21), // "signalsViewSelectDate"
QT_MOC_LITERAL(2, 33, 0), // ""
QT_MOC_LITERAL(3, 34, 4), // "date"
QT_MOC_LITERAL(4, 39, 16), // "signalAngleDelta"
QT_MOC_LITERAL(5, 56, 5), // "delta"
QT_MOC_LITERAL(6, 62, 19), // "signalSwitchPrePage"
QT_MOC_LITERAL(7, 82, 20), // "signalSwitchNextPage"
QT_MOC_LITERAL(8, 103, 15), // "setLunarVisible"
QT_MOC_LITERAL(9, 119, 7), // "visible"
QT_MOC_LITERAL(10, 127, 17), // "setSelectSchedule"
QT_MOC_LITERAL(11, 145, 14), // "DSchedule::Ptr"
QT_MOC_LITERAL(12, 160, 12), // "scheduleInfo"
QT_MOC_LITERAL(13, 173, 24), // "slotScheduleRemindWidget"
QT_MOC_LITERAL(14, 198, 6), // "isShow"
QT_MOC_LITERAL(15, 205, 3) // "out"

    },
    "CMonthView\0signalsViewSelectDate\0\0"
    "date\0signalAngleDelta\0delta\0"
    "signalSwitchPrePage\0signalSwitchNextPage\0"
    "setLunarVisible\0visible\0setSelectSchedule\0"
    "DSchedule::Ptr\0scheduleInfo\0"
    "slotScheduleRemindWidget\0isShow\0out"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CMonthView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   54,    2, 0x06 /* Public */,
       4,    1,   57,    2, 0x06 /* Public */,
       6,    0,   60,    2, 0x06 /* Public */,
       7,    0,   61,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       8,    1,   62,    2, 0x0a /* Public */,
      10,    1,   65,    2, 0x0a /* Public */,
      13,    2,   68,    2, 0x0a /* Public */,
      13,    1,   73,    2, 0x2a /* Public | MethodCloned */,

 // signals: parameters
    QMetaType::Void, QMetaType::QDate,    3,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    9,
    QMetaType::Void, 0x80000000 | 11,   12,
    QMetaType::Void, QMetaType::Bool, 0x80000000 | 11,   14,   15,
    QMetaType::Void, QMetaType::Bool,   14,

       0        // eod
};

void CMonthView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CMonthView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->signalsViewSelectDate((*reinterpret_cast< QDate(*)>(_a[1]))); break;
        case 1: _t->signalAngleDelta((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->signalSwitchPrePage(); break;
        case 3: _t->signalSwitchNextPage(); break;
        case 4: _t->setLunarVisible((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->setSelectSchedule((*reinterpret_cast< const DSchedule::Ptr(*)>(_a[1]))); break;
        case 6: _t->slotScheduleRemindWidget((*reinterpret_cast< const bool(*)>(_a[1])),(*reinterpret_cast< const DSchedule::Ptr(*)>(_a[2]))); break;
        case 7: _t->slotScheduleRemindWidget((*reinterpret_cast< const bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CMonthView::*)(QDate );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CMonthView::signalsViewSelectDate)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (CMonthView::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CMonthView::signalAngleDelta)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (CMonthView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CMonthView::signalSwitchPrePage)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (CMonthView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CMonthView::signalSwitchNextPage)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject CMonthView::staticMetaObject = { {
    QMetaObject::SuperData::link<DWidget::staticMetaObject>(),
    qt_meta_stringdata_CMonthView.data,
    qt_meta_data_CMonthView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CMonthView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CMonthView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CMonthView.stringdata0))
        return static_cast<void*>(this);
    return DWidget::qt_metacast(_clname);
}

int CMonthView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void CMonthView::signalsViewSelectDate(QDate _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void CMonthView::signalAngleDelta(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void CMonthView::signalSwitchPrePage()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void CMonthView::signalSwitchNextPage()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
