/****************************************************************************
** Meta object code from reading C++ file 'dbustimedate.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/dbustimedate.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dbustimedate.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_DBusTimedate_t {
    QByteArrayData data[12];
    char stringdata0[162];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DBusTimedate_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DBusTimedate_t qt_meta_stringdata_DBusTimedate = {
    {
QT_MOC_LITERAL(0, 0, 12), // "DBusTimedate"
QT_MOC_LITERAL(1, 13, 22), // "ShortDateFormatChanged"
QT_MOC_LITERAL(2, 36, 0), // ""
QT_MOC_LITERAL(3, 37, 5), // "value"
QT_MOC_LITERAL(4, 43, 22), // "ShortTimeFormatChanged"
QT_MOC_LITERAL(5, 66, 17), // "WeekBeginsChanged"
QT_MOC_LITERAL(6, 84, 17), // "propertiesChanged"
QT_MOC_LITERAL(7, 102, 12), // "QDBusMessage"
QT_MOC_LITERAL(8, 115, 3), // "msg"
QT_MOC_LITERAL(9, 119, 15), // "ShortTimeFormat"
QT_MOC_LITERAL(10, 135, 15), // "ShortDateFormat"
QT_MOC_LITERAL(11, 151, 10) // "WeekBegins"

    },
    "DBusTimedate\0ShortDateFormatChanged\0"
    "\0value\0ShortTimeFormatChanged\0"
    "WeekBeginsChanged\0propertiesChanged\0"
    "QDBusMessage\0msg\0ShortTimeFormat\0"
    "ShortDateFormat\0WeekBegins"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DBusTimedate[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       4,   14, // methods
       3,   46, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   34,    2, 0x06 /* Public */,
       4,    1,   37,    2, 0x06 /* Public */,
       5,    1,   40,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    1,   43,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    3,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 7,    8,

 // properties: name, type, flags
       9, QMetaType::Int, 0x00495001,
      10, QMetaType::Int, 0x00495001,
      11, QMetaType::Int, 0x00495001,

 // properties: notify_signal_id
       1,
       0,
       2,

       0        // eod
};

void DBusTimedate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DBusTimedate *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->ShortDateFormatChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->ShortTimeFormatChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->WeekBeginsChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->propertiesChanged((*reinterpret_cast< const QDBusMessage(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusMessage >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DBusTimedate::*)(int ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DBusTimedate::ShortDateFormatChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (DBusTimedate::*)(int ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DBusTimedate::ShortTimeFormatChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (DBusTimedate::*)(int ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DBusTimedate::WeekBeginsChanged)) {
                *result = 2;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<DBusTimedate *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< int*>(_v) = _t->shortTimeFormat(); break;
        case 1: *reinterpret_cast< int*>(_v) = _t->shortDateFormat(); break;
        case 2: *reinterpret_cast< int*>(_v) = _t->weekBegins(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject DBusTimedate::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_meta_stringdata_DBusTimedate.data,
    qt_meta_data_DBusTimedate,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DBusTimedate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DBusTimedate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DBusTimedate.stringdata0))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int DBusTimedate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 3;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void DBusTimedate::ShortDateFormatChanged(int _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< DBusTimedate *>(this), &staticMetaObject, 0, _a);
}

// SIGNAL 1
void DBusTimedate::ShortTimeFormatChanged(int _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< DBusTimedate *>(this), &staticMetaObject, 1, _a);
}

// SIGNAL 2
void DBusTimedate::WeekBeginsChanged(int _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< DBusTimedate *>(this), &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
