/****************************************************************************
** Meta object code from reading C++ file 'sidebaritemwidget.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/widget/sidebarWidget/sidebaritemwidget.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'sidebaritemwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_SidebarItemWidget_t {
    QByteArrayData data[5];
    char stringdata0[48];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SidebarItemWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SidebarItemWidget_t qt_meta_stringdata_SidebarItemWidget = {
    {
QT_MOC_LITERAL(0, 0, 17), // "SidebarItemWidget"
QT_MOC_LITERAL(1, 18, 18), // "signalStatusChange"
QT_MOC_LITERAL(2, 37, 0), // ""
QT_MOC_LITERAL(3, 38, 6), // "status"
QT_MOC_LITERAL(4, 45, 2) // "id"

    },
    "SidebarItemWidget\0signalStatusChange\0"
    "\0status\0id"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SidebarItemWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   24,    2, 0x06 /* Public */,
       1,    1,   29,    2, 0x26 /* Public | MethodCloned */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool, QMetaType::QString,    3,    4,
    QMetaType::Void, QMetaType::Bool,    3,

       0        // eod
};

void SidebarItemWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SidebarItemWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->signalStatusChange((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< QString(*)>(_a[2]))); break;
        case 1: _t->signalStatusChange((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SidebarItemWidget::*)(bool , QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SidebarItemWidget::signalStatusChange)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject SidebarItemWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_SidebarItemWidget.data,
    qt_meta_data_SidebarItemWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SidebarItemWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SidebarItemWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SidebarItemWidget.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int SidebarItemWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void SidebarItemWidget::signalStatusChange(bool _t1, QString _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_SidebarTypeItemWidget_t {
    QByteArrayData data[1];
    char stringdata0[22];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SidebarTypeItemWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SidebarTypeItemWidget_t qt_meta_stringdata_SidebarTypeItemWidget = {
    {
QT_MOC_LITERAL(0, 0, 21) // "SidebarTypeItemWidget"

    },
    "SidebarTypeItemWidget"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SidebarTypeItemWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void SidebarTypeItemWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject SidebarTypeItemWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<SidebarItemWidget::staticMetaObject>(),
    qt_meta_stringdata_SidebarTypeItemWidget.data,
    qt_meta_data_SidebarTypeItemWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SidebarTypeItemWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SidebarTypeItemWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SidebarTypeItemWidget.stringdata0))
        return static_cast<void*>(this);
    return SidebarItemWidget::qt_metacast(_clname);
}

int SidebarTypeItemWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SidebarItemWidget::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_SidebarAccountItemWidget_t {
    QByteArrayData data[9];
    char stringdata0[175];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SidebarAccountItemWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SidebarAccountItemWidget_t qt_meta_stringdata_SidebarAccountItemWidget = {
    {
QT_MOC_LITERAL(0, 0, 24), // "SidebarAccountItemWidget"
QT_MOC_LITERAL(1, 25, 19), // "slotRearIconClicked"
QT_MOC_LITERAL(2, 45, 0), // ""
QT_MOC_LITERAL(3, 46, 20), // "slotSyncStatusChange"
QT_MOC_LITERAL(4, 67, 26), // "DAccount::AccountSyncState"
QT_MOC_LITERAL(5, 94, 22), // "slotAccountStateChange"
QT_MOC_LITERAL(6, 117, 22), // "slotNetworkStateChange"
QT_MOC_LITERAL(7, 140, 28), // "DOANetWorkDBus::NetWorkState"
QT_MOC_LITERAL(8, 169, 5) // "state"

    },
    "SidebarAccountItemWidget\0slotRearIconClicked\0"
    "\0slotSyncStatusChange\0DAccount::AccountSyncState\0"
    "slotAccountStateChange\0slotNetworkStateChange\0"
    "DOANetWorkDBus::NetWorkState\0state"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SidebarAccountItemWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       4,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   34,    2, 0x0a /* Public */,
       3,    1,   35,    2, 0x0a /* Public */,
       5,    0,   38,    2, 0x0a /* Public */,
       6,    1,   39,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 7,    8,

       0        // eod
};

void SidebarAccountItemWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SidebarAccountItemWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->slotRearIconClicked(); break;
        case 1: _t->slotSyncStatusChange((*reinterpret_cast< DAccount::AccountSyncState(*)>(_a[1]))); break;
        case 2: _t->slotAccountStateChange(); break;
        case 3: _t->slotNetworkStateChange((*reinterpret_cast< DOANetWorkDBus::NetWorkState(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject SidebarAccountItemWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<SidebarItemWidget::staticMetaObject>(),
    qt_meta_stringdata_SidebarAccountItemWidget.data,
    qt_meta_data_SidebarAccountItemWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SidebarAccountItemWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SidebarAccountItemWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SidebarAccountItemWidget.stringdata0))
        return static_cast<void*>(this);
    return SidebarItemWidget::qt_metacast(_clname);
}

int SidebarAccountItemWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SidebarItemWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 4;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
