/****************************************************************************
** Meta object code from reading C++ file 'scheduleview.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/customWidget/scheduleview.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QVector>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'scheduleview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_CScheduleView_t {
    QByteArrayData data[29];
    char stringdata0[389];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CScheduleView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CScheduleView_t qt_meta_stringdata_CScheduleView = {
    {
QT_MOC_LITERAL(0, 0, 13), // "CScheduleView"
QT_MOC_LITERAL(1, 14, 26), // "signalsCurrentScheduleDate"
QT_MOC_LITERAL(2, 41, 0), // ""
QT_MOC_LITERAL(3, 42, 4), // "date"
QT_MOC_LITERAL(4, 47, 16), // "signalAngleDelta"
QT_MOC_LITERAL(5, 64, 5), // "delta"
QT_MOC_LITERAL(6, 70, 19), // "signalSwitchPrePage"
QT_MOC_LITERAL(7, 90, 20), // "signalSwitchNextPage"
QT_MOC_LITERAL(8, 111, 12), // "slotPosHours"
QT_MOC_LITERAL(9, 124, 12), // "QVector<int>"
QT_MOC_LITERAL(10, 137, 4), // "vPos"
QT_MOC_LITERAL(11, 142, 6), // "vHours"
QT_MOC_LITERAL(12, 149, 15), // "currentTimeType"
QT_MOC_LITERAL(13, 165, 7), // "setDate"
QT_MOC_LITERAL(14, 173, 18), // "slotupdateSchedule"
QT_MOC_LITERAL(15, 192, 14), // "slotDeleteitem"
QT_MOC_LITERAL(16, 207, 23), // "slotCurrentScheduleDate"
QT_MOC_LITERAL(17, 231, 16), // "slotScheduleShow"
QT_MOC_LITERAL(18, 248, 6), // "isShow"
QT_MOC_LITERAL(19, 255, 14), // "DSchedule::Ptr"
QT_MOC_LITERAL(20, 270, 3), // "out"
QT_MOC_LITERAL(21, 274, 15), // "slotUpdatePaint"
QT_MOC_LITERAL(22, 290, 4), // "topM"
QT_MOC_LITERAL(23, 295, 15), // "slotUpdateScene"
QT_MOC_LITERAL(24, 311, 14), // "slotSwitchView"
QT_MOC_LITERAL(25, 326, 9), // "focusDate"
QT_MOC_LITERAL(26, 336, 30), // "CWeekDayGraphicsview::ViewType"
QT_MOC_LITERAL(27, 367, 8), // "viewtype"
QT_MOC_LITERAL(28, 376, 12) // "setItemFocus"

    },
    "CScheduleView\0signalsCurrentScheduleDate\0"
    "\0date\0signalAngleDelta\0delta\0"
    "signalSwitchPrePage\0signalSwitchNextPage\0"
    "slotPosHours\0QVector<int>\0vPos\0vHours\0"
    "currentTimeType\0setDate\0slotupdateSchedule\0"
    "slotDeleteitem\0slotCurrentScheduleDate\0"
    "slotScheduleShow\0isShow\0DSchedule::Ptr\0"
    "out\0slotUpdatePaint\0topM\0slotUpdateScene\0"
    "slotSwitchView\0focusDate\0"
    "CWeekDayGraphicsview::ViewType\0viewtype\0"
    "setItemFocus"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CScheduleView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      15,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   89,    2, 0x06 /* Public */,
       4,    1,   92,    2, 0x06 /* Public */,
       6,    0,   95,    2, 0x06 /* Public */,
       7,    0,   96,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       8,    3,   97,    2, 0x0a /* Public */,
       8,    2,  104,    2, 0x2a /* Public | MethodCloned */,
      13,    1,  109,    2, 0x0a /* Public */,
      14,    0,  112,    2, 0x0a /* Public */,
      15,    0,  113,    2, 0x0a /* Public */,
      16,    1,  114,    2, 0x0a /* Public */,
      17,    2,  117,    2, 0x0a /* Public */,
      17,    1,  122,    2, 0x2a /* Public | MethodCloned */,
      21,    1,  125,    2, 0x0a /* Public */,
      23,    0,  128,    2, 0x0a /* Public */,
      24,    3,  129,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QDate,    3,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 9, 0x80000000 | 9, QMetaType::Int,   10,   11,   12,
    QMetaType::Void, 0x80000000 | 9, 0x80000000 | 9,   10,   11,
    QMetaType::Void, QMetaType::QDate,    3,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QDate,    3,
    QMetaType::Void, QMetaType::Bool, 0x80000000 | 19,   18,   20,
    QMetaType::Void, QMetaType::Bool,   18,
    QMetaType::Void, QMetaType::Int,   22,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QDate, 0x80000000 | 26, QMetaType::Bool,   25,   27,   28,

       0        // eod
};

void CScheduleView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CScheduleView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->signalsCurrentScheduleDate((*reinterpret_cast< QDate(*)>(_a[1]))); break;
        case 1: _t->signalAngleDelta((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->signalSwitchPrePage(); break;
        case 3: _t->signalSwitchNextPage(); break;
        case 4: _t->slotPosHours((*reinterpret_cast< QVector<int>(*)>(_a[1])),(*reinterpret_cast< QVector<int>(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 5: _t->slotPosHours((*reinterpret_cast< QVector<int>(*)>(_a[1])),(*reinterpret_cast< QVector<int>(*)>(_a[2]))); break;
        case 6: _t->setDate((*reinterpret_cast< QDate(*)>(_a[1]))); break;
        case 7: _t->slotupdateSchedule(); break;
        case 8: _t->slotDeleteitem(); break;
        case 9: _t->slotCurrentScheduleDate((*reinterpret_cast< QDate(*)>(_a[1]))); break;
        case 10: _t->slotScheduleShow((*reinterpret_cast< const bool(*)>(_a[1])),(*reinterpret_cast< const DSchedule::Ptr(*)>(_a[2]))); break;
        case 11: _t->slotScheduleShow((*reinterpret_cast< const bool(*)>(_a[1]))); break;
        case 12: _t->slotUpdatePaint((*reinterpret_cast< const int(*)>(_a[1]))); break;
        case 13: _t->slotUpdateScene(); break;
        case 14: _t->slotSwitchView((*reinterpret_cast< const QDate(*)>(_a[1])),(*reinterpret_cast< CWeekDayGraphicsview::ViewType(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QVector<int> >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QVector<int> >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CScheduleView::*)(QDate );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CScheduleView::signalsCurrentScheduleDate)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (CScheduleView::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CScheduleView::signalAngleDelta)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (CScheduleView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CScheduleView::signalSwitchPrePage)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (CScheduleView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CScheduleView::signalSwitchNextPage)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject CScheduleView::staticMetaObject = { {
    QMetaObject::SuperData::link<DFrame::staticMetaObject>(),
    qt_meta_stringdata_CScheduleView.data,
    qt_meta_data_CScheduleView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CScheduleView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CScheduleView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CScheduleView.stringdata0))
        return static_cast<void*>(this);
    return DFrame::qt_metacast(_clname);
}

int CScheduleView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void CScheduleView::signalsCurrentScheduleDate(QDate _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void CScheduleView::signalAngleDelta(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void CScheduleView::signalSwitchPrePage()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void CScheduleView::signalSwitchNextPage()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
