/****************************************************************************
** Meta object code from reading C++ file 'monthscheduleview.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/widget/monthWidget/monthscheduleview.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'monthscheduleview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_CMonthScheduleView_t {
    QByteArrayData data[13];
    char stringdata0[162];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CMonthScheduleView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CMonthScheduleView_t qt_meta_stringdata_CMonthScheduleView = {
    {
QT_MOC_LITERAL(0, 0, 18), // "CMonthScheduleView"
QT_MOC_LITERAL(1, 19, 21), // "signalsUpdateSchedule"
QT_MOC_LITERAL(2, 41, 0), // ""
QT_MOC_LITERAL(3, 42, 2), // "id"
QT_MOC_LITERAL(4, 45, 26), // "signalsCurrentScheduleDate"
QT_MOC_LITERAL(5, 72, 4), // "date"
QT_MOC_LITERAL(6, 77, 14), // "signalUpdateUI"
QT_MOC_LITERAL(7, 92, 4), // "type"
QT_MOC_LITERAL(8, 97, 23), // "signalPressScheduleShow"
QT_MOC_LITERAL(9, 121, 6), // "isShow"
QT_MOC_LITERAL(10, 128, 14), // "DSchedule::Ptr"
QT_MOC_LITERAL(11, 143, 3), // "out"
QT_MOC_LITERAL(12, 147, 14) // "slotFontChange"

    },
    "CMonthScheduleView\0signalsUpdateSchedule\0"
    "\0id\0signalsCurrentScheduleDate\0date\0"
    "signalUpdateUI\0type\0signalPressScheduleShow\0"
    "isShow\0DSchedule::Ptr\0out\0slotFontChange"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CMonthScheduleView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       6,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   49,    2, 0x06 /* Public */,
       1,    0,   52,    2, 0x26 /* Public | MethodCloned */,
       4,    1,   53,    2, 0x06 /* Public */,
       6,    1,   56,    2, 0x06 /* Public */,
       8,    2,   59,    2, 0x06 /* Public */,
       8,    1,   64,    2, 0x26 /* Public | MethodCloned */,

 // slots: name, argc, parameters, tag, flags
      12,    0,   67,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QDate,    5,
    QMetaType::Void, QMetaType::Int,    7,
    QMetaType::Void, QMetaType::Bool, 0x80000000 | 10,    9,   11,
    QMetaType::Void, QMetaType::Bool,    9,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void CMonthScheduleView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CMonthScheduleView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->signalsUpdateSchedule((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->signalsUpdateSchedule(); break;
        case 2: _t->signalsCurrentScheduleDate((*reinterpret_cast< QDate(*)>(_a[1]))); break;
        case 3: _t->signalUpdateUI((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->signalPressScheduleShow((*reinterpret_cast< const bool(*)>(_a[1])),(*reinterpret_cast< const DSchedule::Ptr(*)>(_a[2]))); break;
        case 5: _t->signalPressScheduleShow((*reinterpret_cast< const bool(*)>(_a[1]))); break;
        case 6: _t->slotFontChange(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CMonthScheduleView::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CMonthScheduleView::signalsUpdateSchedule)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (CMonthScheduleView::*)(QDate );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CMonthScheduleView::signalsCurrentScheduleDate)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (CMonthScheduleView::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CMonthScheduleView::signalUpdateUI)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (CMonthScheduleView::*)(const bool , const DSchedule::Ptr & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CMonthScheduleView::signalPressScheduleShow)) {
                *result = 4;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject CMonthScheduleView::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_CMonthScheduleView.data,
    qt_meta_data_CMonthScheduleView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CMonthScheduleView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CMonthScheduleView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CMonthScheduleView.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int CMonthScheduleView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void CMonthScheduleView::signalsUpdateSchedule(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 2
void CMonthScheduleView::signalsCurrentScheduleDate(QDate _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void CMonthScheduleView::signalUpdateUI(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void CMonthScheduleView::signalPressScheduleShow(const bool _t1, const DSchedule::Ptr & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
struct qt_meta_stringdata_CWeekScheduleView_t {
    QByteArrayData data[1];
    char stringdata0[18];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CWeekScheduleView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CWeekScheduleView_t qt_meta_stringdata_CWeekScheduleView = {
    {
QT_MOC_LITERAL(0, 0, 17) // "CWeekScheduleView"

    },
    "CWeekScheduleView"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CWeekScheduleView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void CWeekScheduleView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject CWeekScheduleView::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_CWeekScheduleView.data,
    qt_meta_data_CWeekScheduleView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CWeekScheduleView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CWeekScheduleView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CWeekScheduleView.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int CWeekScheduleView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
